@@ -253,7 +253,7 @@ class DeviceManager(object):
     def shutdown(self):
         self.running = False
         for device in self.connected.values():
-            if device.mount:
+            if device.mount and not self._is_hidden(device):
                 write_database(device.database, device.mount)
 
     def load_devices(self, path):
@@ -478,10 +478,6 @@ class DeviceSyncManager(object):
         self.waiting = set()
         self.stopping = False
 
-        self.device.is_updating = True # start the spinner
-        messages.TabsChanged('connect', [], [self.device],
-                             []).send_to_frontend()
-
         self.audio_target_folder = os.path.join(
             device.mount,
             self._get_path_from_setting('audio_path'))
@@ -494,6 +490,10 @@ class DeviceSyncManager(object):
         if not os.path.exists(self.video_target_folder):
             os.makedirs(self.video_target_folder)
 
+        self.device.is_updating = True # start the spinner
+        messages.TabsChanged('connect', [], [self.device],
+                             []).send_to_frontend()
+
     def _get_path_from_setting(self, setting):
         device_settings = self.device.database.setdefault(u'settings', {})
         device_path = device_settings.get(setting)
@@ -675,7 +675,7 @@ class DeviceSyncManager(object):
                 )
             device_item._migrate_thumbnail()
             database = self.device.database
-            database.setdefault(device_item.file_type, [])
+            database.setdefault(device_item.file_type, {})
             database[device_item.file_type][device_item.id] = \
                 device_item.to_dict()
             database.emit('item-added', device_item)
@@ -1072,6 +1072,9 @@ def clean_database(device):
     to_remove = []
     for item_type in (u'video', u'audio', u'other'):
         device.database.setdefault(item_type, {})
+        if isinstance(device.database[item_type], list):
+            # 17554: we could accidentally set this to a list
+            device.database[item_type] = {}
         for item_path_unicode in device.database[item_type]:
             item_path = utf8_to_filename(item_path_unicode.encode('utf8'))
             if _exists(item_path):
