@@ -71,8 +71,13 @@ from miro.plat.filebundle import is_file_bundle
 from miro import filetypes
 from miro.item import FeedParserValues
 from miro import searchengines
+from miro import workerprocess
 from miro.clock import clock
 
+# should we not use our worker process for feedparser.  This is just used in
+# the unittests to speed things up
+_RUN_FEED_PARSER_INLINE = False
+
 WHITESPACE_PATTERN = re.compile(r"^[ \t\r\n]*$")
 
 DEFAULT_FEED_ICON = "images/icon-podcast-small.png"
@@ -84,6 +89,29 @@ def default_feed_icon_url():
 def default_feed_icon_path():
     return resources.path(DEFAULT_FEED_ICON)
 
+class _RateLimiter(object):
+    """Helper class used by create_items_for_parsed() to avoid hogging the
+    GIL.
+
+    This class helps ensure that a potentially long-running operation doesn't
+    take more than a fixed-percentage of the CPU.  It's not really a great
+    solution, but it's the simplest way for 4.0.2
+    """
+    def __init__(self):
+        self.last_time = time.time()
+
+    def check_for_sleep(self):
+        new_time = time.time()
+        elapsed = self.last_time - time.time()
+        if elapsed < 0.1:
+            return # don't sleep until a decent of time has passed.
+        # We want to yield at least 10% of the CPU.  Sleep for 10% of the time
+        # we took since the last check
+        sleep_time = elapsed / 10.0
+        time.sleep(sleep_time)
+        # get ready for the next check() call
+        self.last_time = time.time()
+
 # Notes on character set encoding of feeds:
 #
 # The parsing libraries built into Python mostly use byte strings
@@ -200,6 +228,17 @@ def _on_config_change(obj, key, value):
                 pass
             feed.set_update_frequency(update_freq)
 
+def run_feedparser(html, callback, errback):
+    if _RUN_FEED_PARSER_INLINE:
+        try:
+            rv = feedparserutil.parse(html)
+        except StandardError, e:
+            errback(e)
+        else:
+            callback(rv)
+    else:
+        workerprocess.run_feedparser(html, callback, errback)
+
 # Wait X seconds before updating the feeds at startup
 INITIAL_FEED_UPDATE_DELAY = 5.0
 
@@ -1207,25 +1246,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
             app.bulk_sql_manager.finish()
 
     def _create_items_for_parsed(self, parsed):
-        # This is a HACK for Yahoo! search which doesn't provide
-        # enclosures
-        for entry in parsed['entries']:
-            if 'enclosures' not in entry:
-                try:
-                    url = entry['link']
-                except KeyError:
-                    continue
-                mimetype = filetypes.guess_mime_type(url)
-                if mimetype is not None:
-                    entry['enclosures'] = [{'url': to_uni(url),
-                                            'type': to_uni(mimetype)}]
-                elif flashscraper.is_maybe_flashscrapable(url):
-                    entry['enclosures'] = [{'url': to_uni(url),
-                                            'type': to_uni("video/flv")}]
-                else:
-                    logging.warning(
-                        'unknown url type %s, not generating enclosure', url)
-
+        rate_limiter = _RateLimiter()
         channel_title = None
         try:
             channel_title = parsed["feed"]["title"]
@@ -1247,6 +1268,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
         items_byURLTitle = {}
         items_nokey = []
         for item in self.items:
+            rate_limiter.check_for_sleep()
             try:
                 items_byid[item.get_rss_id()] = item
             except KeyError:
@@ -1255,6 +1277,7 @@ class RSSFeedImplBase(ThrottledUpdateFeedImpl):
             if by_url_title_key != (None, None):
                 items_byURLTitle[by_url_title_key] = item
         for entry in parsed.entries:
+            rate_limiter.check_for_sleep()
             entry = self.add_scraped_thumbnail(entry)
             fp_values = FeedParserValues(entry)
             new = True
@@ -1411,14 +1434,14 @@ class RSSFeedImpl(RSSFeedImplBase):
             self.feedparser_finished()
             return
         start = clock()
-        parsed = self.parsed = unicodify(parsed)
+        self.parsed = parsed
         self.remember_old_items()
         self.create_items_for_parsed(parsed)
 
         try:
             updateFreq = self.parsed["feed"]["ttl"]
         except KeyError:
-            updateFreq = -1
+            updateFreq = 0
         self.set_update_frequency(updateFreq)
 
         self.feedparser_finished()
@@ -1429,7 +1452,7 @@ class RSSFeedImpl(RSSFeedImplBase):
 
     def call_feedparser(self, html):
         self.ufeed.confirm_db_thread()
-        feedparserutil.queue_parse(html, self.feedparser_callback,
+        run_feedparser(html, self.feedparser_callback,
                 self.feedparser_errback)
 
     def update(self):
@@ -1577,7 +1600,6 @@ class RSSMultiFeedBase(RSSFeedImplBase):
         if not self.ufeed.id_exists() or url not in self.download_dc:
             return
         start = clock()
-        parsed = unicodify(parsed)
         self.create_items_for_parsed(parsed)
         self.feedparser_finished(url)
         end = clock()
@@ -1587,7 +1609,7 @@ class RSSMultiFeedBase(RSSFeedImplBase):
 
     def call_feedparser(self, html, url):
         self.ufeed.confirm_db_thread()
-        feedparserutil.queue_parse(html,
+        run_feedparser(html,
             lambda parsed, url=url: self.feedparser_callback(parsed, url),
             lambda e, url=url: self.feedparser_errback(e, url))
 
@@ -2172,7 +2194,7 @@ class DirectoryScannerImplBase(FeedImpl):
             yield # yield after doing prep work
             try:
                 while not finished:
-                    finished = self._add_batch_of_videos(path_iter, 0.5)
+                    finished = self._add_batch_of_videos(path_iter, 0.2)
                     yield # yield after each batch
             except ObjectNotFoundError:
                 # whoops, we disappeared! clean up and quit
