@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,18 +37,23 @@ import os
 from miro import app
 from miro import messages
 from miro import signals
-from miro import config
 from miro import prefs
 from miro import filetypes
 from miro.gtcache import gettext as _
 from miro.gtcache import ngettext
 from miro.frontends.widgets import browser
 from miro.frontends.widgets import downloadscontroller
-from miro.frontends.widgets import videoconversionscontroller
+from miro.frontends.widgets import convertingcontroller
 from miro.frontends.widgets import feedcontroller
+from miro.frontends.widgets import guidecontroller
 from miro.frontends.widgets import itemlistcontroller
+from miro.frontends.widgets import devicecontroller
+from miro.frontends.widgets import sharingcontroller
+from miro.frontends.widgets import searchcontroller
+from miro.frontends.widgets import tabcontroller
 from miro.frontends.widgets import playlist
 from miro.frontends.widgets import widgetutil
+from miro.plat.frontends.widgets.threads import call_on_ui_thread
 from miro.plat.frontends.widgets import widgetset
 
 class Display(signals.SignalEmitter):
@@ -69,9 +75,12 @@ class Display(signals.SignalEmitter):
         """
         pass
 
-    def on_activate(self):
+    def on_activate(self, is_push):
         """Perform code that needs to be run when the display becomes the
         active display (the one on the top of the display stack).
+
+        :param is_push: are we pushing the display on top (as opposed to
+            having a display that was on top of us get popped off)
         """
         pass
 
@@ -93,7 +102,7 @@ class TabDisplay(Display):
     def __init__(self, tab_type, selected_tabs):
         raise NotImplementedError()
 
-    def on_activate(self):
+    def on_activate(self, is_push):
         app.menu_manager.update_menus()
 
     @staticmethod
@@ -111,25 +120,42 @@ class DisplayManager(object):
     when the playback is finished.
     """
     def __init__(self):
-        self.display_classes = [
-                AudioFeedDisplay,
+        # displays that we construct when the user clicks on them
+        self.on_demand_display_classes = [
                 FeedDisplay,
+                AllFeedsDisplay,
                 PlaylistDisplay,
                 SiteDisplay,
                 SearchDisplay,
-                VideoItemsDisplay,
-                AudioItemsDisplay,
                 OtherItemsDisplay,
                 DownloadingDisplay,
-                VideoConversionsDisplay,
+                ConvertingDisplay,
                 GuideDisplay,
                 MultipleSelectionDisplay,
+                DeviceDisplay,
+                DeviceItemDisplay,
+                SharingDisplay,
+                ConnectDisplay,
+                SourcesDisplay,
+                PlaylistsDisplay,
+                StoresDisplay,
+
+                # DummyDisplay should be last because it's a
+                # catch-all.
                 DummyDisplay,
         ]
+        # displays that we keep alive all the time
+        self.permanent_displays = set()
+        self.add_permanent_display(VideoItemsDisplay())
+        self.add_permanent_display(AudioItemsDisplay())
         self.display_stack = []
         self.selected_tab_list = self.selected_tabs = None
         app.info_updater.connect('sites-removed', SiteDisplay.on_sites_removed)
 
+    def add_permanent_display(self, display):
+        self.permanent_displays.add(display)
+        display.on_selected()
+
     def get_current_display(self):
         try:
             return self.display_stack[-1]
@@ -143,16 +169,25 @@ class DisplayManager(object):
                 selected_tabs == self.selected_tabs and
                 len(self.display_stack) > 0 and
                 isinstance(self.display_stack[-1], TabDisplay)):
-            logging.debug('not reselecting')
+            logging.warn('not reselecting')
             return
 
         self.selected_tab_list = selected_tab_list
         self.selected_tabs = selected_tabs
-        tab_type = selected_tab_list.type
+        # parents always come first, so using the type of the last item handles
+        # the case where a root item and its child(ren) are selected
+        if hasattr(selected_tabs[-1], 'type'):
+            tab_type = selected_tabs[-1].type
+        else:
+            tab_type = selected_tab_list.type
 
-        for klass in self.display_classes:
+        for display in self.permanent_displays:
+            if display.should_display(tab_type, selected_tabs):
+                self.change_non_video_displays(display)
+                return
+        for klass in self.on_demand_display_classes:
             if klass.should_display(tab_type, selected_tabs):
-                self.select_display(klass(tab_type, selected_tabs))
+                self.change_non_video_displays(klass(tab_type, selected_tabs))
                 return
         raise AssertionError(
             "Can't find display for %s %s" % (tab_type, selected_tabs))
@@ -162,19 +197,53 @@ class DisplayManager(object):
         self.deselect_all_displays()
         self.push_display(display)
 
+    def change_non_video_displays(self, display):
+        """Like select_display(), but don't replace the VideoDisplay
+
+        Mostly this will work like select_display().  However, if there is a
+        VideoDisplay on top of the display, it will stay on top.
+
+        The main reason for this method is when we are playing video and the
+        current tab gets removed (#16225).  In this case, we want to select a
+        new tab and make a display for that tab, but not show that display
+        until video stops
+        """
+
+        if (len(self.display_stack) == 0 or
+                not isinstance(self.display_stack[-1], VideoDisplay)):
+            # no video displays are on the stack, just call select_display
+            self.select_display(display)
+            return
+        # save the video display object
+        video_display = self.display_stack.pop()
+        # unselect displays below it
+        for old_display in self.display_stack:
+            self._unselect_display(old_display, on_top=False)
+        # re-create the display stack with the new display and the video
+        # display on top of it
+        self.display_stack = [display, video_display]
+        # call on_selected() if we are creating the display.  Don't call
+        # on_activate() or show the display because it's still below other
+        # displays
+        if display not in self.permanent_displays:
+            display.on_selected()
+
     def deselect_all_displays(self):
         """Deselect all displays."""
-        for old_display in self.display_stack:
-            self._unselect_display(old_display)
-        self.display_stack = []
+        top_display = True
+        while self.display_stack:
+            old_display = self.display_stack.pop()
+            self._unselect_display(old_display, on_top=top_display)
+            top_display = False
 
     def push_display(self, display):
         """Select a display and push it on top of the display stack"""
         if len(self.display_stack) > 0:
             self.current_display.on_deactivate()
         self.display_stack.append(display)
-        display.on_selected()
-        display.on_activate()
+        if display not in self.permanent_displays:
+            display.on_selected()
+        display.on_activate(is_push=True)
         app.widgetapp.window.set_main_area(display.widget)
 
     def pop_display(self, unselect=True):
@@ -183,17 +252,19 @@ class DisplayManager(object):
         """
         display = self.display_stack.pop()
         if unselect:
-            self._unselect_display(display)
-        self.current_display.on_activate()
+            self._unselect_display(display, on_top=True)
+        self.current_display.on_activate(is_push=False)
         app.widgetapp.window.set_main_area(self.current_display.widget)
 
-    def _unselect_display(self, display):
-        display.on_deactivate()
-        display.cleanup()
+    def _unselect_display(self, display, on_top):
+        if on_top:
+            display.on_deactivate()
+        if display not in self.permanent_displays:
+            display.cleanup()
         display.emit("removed")
 
-    def push_folder_contents_display(self, folder_info):
-        self.push_display(FolderContentsDisplay(folder_info))
+    def push_folder_contents_display(self, folder_info, start_playing=False):
+        self.push_display(FolderContentsDisplay(folder_info, start_playing))
 
 class GuideDisplay(TabDisplay):
     @staticmethod
@@ -202,22 +273,37 @@ class GuideDisplay(TabDisplay):
 
     def __init__(self, tab_type, selected_tabs):
         Display.__init__(self)
-        self.widget = selected_tabs[0].browser
+        self.widget = guidecontroller.GuideTab(selected_tabs[0].browser)
+        app.info_updater.item_list_callbacks.add(u'guide-sidebar', None,
+                                                 self.on_item_list),
+        app.info_updater.item_changed_callbacks.add(u'guide-sidebar', None,
+                                                    self.on_item_changed),
+        messages.TrackItems(u'guide-sidebar', None).send_to_backend()
+        app.display_manager.add_permanent_display(self) # once we're loaded,
+                                                        # stay loaded
+
+    def on_item_list(self, message):
+        self.widget.on_item_list(message.items)
+
+    def on_item_changed(self, message):
+        self.widget.on_item_changed(message.added, message.changed,
+                                    message.removed)
 
 class SiteDisplay(TabDisplay):
     _open_sites = {} # maps site ids -> BrowserNav objects for them
 
     @classmethod
     def on_sites_removed(cls, info_updater, id_list):
-        for id in id_list:
+        for id_ in id_list:
             try:
-                del cls._open_sites[id]
+                del cls._open_sites[id_]
             except KeyError:
                 pass
 
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'site' and len(selected_tabs) == 1
+        return tab_type in ('site', 'store') and len(selected_tabs) == 1 and \
+               hasattr(selected_tabs[0], 'url')
 
     def __init__(self, tab_type, selected_tabs):
         Display.__init__(self)
@@ -230,11 +316,17 @@ class ItemListDisplayMixin(object):
     def on_selected(self):
         app.item_list_controller_manager.controller_created(self.controller)
         self.controller.start_tracking()
-        self.restore_state()
 
-    def on_activate(self):
+    def on_activate(self, is_push):
         app.item_list_controller_manager.controller_displayed(self.controller)
-        super(ItemListDisplayMixin, self).on_activate()
+        if not is_push:
+            # Focus the item list when we pop the video display from being on
+            # top of us.
+            #
+            # FIXME: call_on_ui_thread is a bit weird here.  It's needed
+            # because on OS X we can't call focus() yet on our tableview.
+            call_on_ui_thread(self.controller.focus_view)
+        super(ItemListDisplayMixin, self).on_activate(is_push)
 
     def on_deactivate(self):
         app.item_list_controller_manager.controller_no_longer_displayed(
@@ -242,18 +334,10 @@ class ItemListDisplayMixin(object):
 
     def cleanup(self):
         self.controller.stop_tracking()
-        self.remember_state()
         app.item_list_controller_manager.controller_destroyed(self.controller)
 
-    def remember_state(self):
-        if self.controller.widget.in_list_view:
-            app.frontend_states_memory.set_list_view(self.type, self.id)
-        else:
-            app.frontend_states_memory.set_std_view(self.type, self.id)
-
-    def restore_state(self):
-        if app.frontend_states_memory.query_list_view(self.type, self.id):
-            self.widget.switch_to_list_view()
+    def update_columns_enabled(self):
+        self.controller.update_columns_enabled()
 
 class ItemListDisplay(ItemListDisplayMixin, TabDisplay):
     def __init__(self, tab_type, selected_tabs):
@@ -268,102 +352,138 @@ class ItemListDisplay(ItemListDisplayMixin, TabDisplay):
         raise NotImplementedError()
 
 class FeedDisplay(ItemListDisplay):
-    TAB_TYPE = 'feed'
+    TAB_TYPE = u'feed'
     UPDATER_SIGNAL_NAME = 'feeds-changed'
 
     @classmethod
     def should_display(cls, tab_type, selected_tabs):
-        return tab_type == cls.TAB_TYPE and len(selected_tabs) == 1
-
-    def on_selected(self):
-        ItemListDisplay.on_selected(self)
-        self._signal_handler = app.info_updater.connect(
-                self.UPDATER_SIGNAL_NAME, self._on_feeds_changed)
-
-    def _on_feeds_changed(self, updater, info_list):
-        for info in info_list:
-            if info.id == self.id:
-                self.controller.titlebar.update_title(info.name)
-                return
+        return tab_type == cls.TAB_TYPE and len(selected_tabs) == 1 and \
+               selected_tabs[0].type != u'tab'
 
     def cleanup(self):
         ItemListDisplay.cleanup(self)
-        app.info_updater.disconnect(self._signal_handler)
         if widgetutil.feed_exists(self.feed_id):
             messages.MarkFeedSeen(self.feed_id).send_to_backend()
 
     def make_controller(self, tab):
         self.feed_id = tab.id
-        return feedcontroller.FeedController(tab.id, tab.is_folder, tab.is_directory_feed)
+        return feedcontroller.FeedController(tab.id, tab.is_folder,
+                                             tab.is_directory_feed)
 
-class AudioFeedDisplay(FeedDisplay):
-    TAB_TYPE = 'audio-feed'
-    UPDATER_SIGNAL_NAME = 'audio-feeds-changed'
+class AllFeedsDisplay(FeedDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return (tab_type == u'tab' and len(selected_tabs) == 1 and
+               selected_tabs[0].tab_class == u'feed')
+
+    def make_controller(self, tab):
+        return feedcontroller.AllFeedsController(tab.id, True, True)
+
+    def cleanup(self):
+        ItemListDisplay.cleanup(self)
 
 class PlaylistDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'playlist' and len(selected_tabs) == 1
-
-    def on_selected(self):
-        ItemListDisplay.on_selected(self)
-        self._signal_handler = app.info_updater.connect('playlists-changed',
-                self._on_playlists_changed)
-
-    def _on_playlists_changed(self, updater, info_list):
-        for info in info_list:
-            if info.id == self.id:
-                self.controller.titlebar.update_title(info.name)
-                return
-
-    def cleanup(self):
-        ItemListDisplay.cleanup(self)
-        app.info_updater.disconnect(self._signal_handler)
+        return tab_type == 'playlist' and len(selected_tabs) == 1 and \
+               selected_tabs[0].type != u'tab'
 
     def make_controller(self, playlist_info):
-        return playlist.PlaylistView(playlist_info)
+        return playlist.PlaylistItemController(playlist_info)
+
+class DeviceDisplayMixin(object):
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+        device = selected_tabs[0]
+        if getattr(device, 'fake', False):
+            self.controller = devicecontroller.DeviceItemController(device)
+            self.type = self.controller.type
+            self.id = device.id
+        else:
+            self.controller = devicecontroller.DeviceController(device)
+        self.widget = self.controller.widget
+
+    def handle_current_sync_information(self, message):
+        return
+
+    def handle_device_sync_changed(self, message):
+        pass
+
+class DeviceDisplay(DeviceDisplayMixin, TabDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == u'device' and len(selected_tabs) == 1 and \
+               isinstance(selected_tabs[0], messages.DeviceInfo) and \
+               not getattr(selected_tabs[0], 'fake', False)
+
+    def on_selected(self):
+        self.controller.start_tracking()
+
+    def cleanup(self):
+        self.controller.stop_tracking()
+
+    def handle_current_sync_information(self, message):
+        if not getattr(self.controller.device, 'fake', False):
+            self.controller.handle_current_sync_information(message)
+
+    def handle_device_sync_changed(self, message):
+        if not getattr(self.controller.device, 'fake', False):
+            self.controller.handle_device_sync_changed(message)
+
+class DeviceItemDisplay(DeviceDisplayMixin, ItemListDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == u'device' and len(selected_tabs) == 1 and \
+               isinstance(selected_tabs[0], messages.DeviceInfo) and \
+               getattr(selected_tabs[0], 'fake', False)
+
+class SharingDisplay(ItemListDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == u'sharing' and len(selected_tabs) == 1
+
+    def make_controller(self, tab):
+        return sharingcontroller.SharingView(tab)
 
 class SearchDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'static' and selected_tabs[0].id == 'search'
+        return tab_type == 'search' and selected_tabs[0].id == 'search'
 
     def make_controller(self, tab):
-        return itemlistcontroller.SearchController()
+        return searchcontroller.SearchController()
 
 class AudioVideoItemsDisplay(ItemListDisplay):
-    def remember_state(self):
-        ItemListDisplay.remember_state(self)
-        filters = self.widget.toolbar.active_filters()
-        app.frontend_states_memory.set_filters(self.type, self.id, filters)
+    def __init__(self):
+        Display.__init__(self)
+        self.controller = self.make_controller()
+        self.widget = self.controller.widget
+        self.type = self.__class__.tab_type
+        self.id = self.__class__.tab_id
 
-    def restore_state(self):
-        initial_filters = app.frontend_states_memory.query_filters(self.type,
-                self.id)
-        if initial_filters:
-            self.controller.set_item_filters(initial_filters)
-        ItemListDisplay.restore_state(self)
+    @classmethod
+    def should_display(cls, tab_type, selected_tabs):
+        return (hasattr(selected_tabs[0], 'type') and selected_tabs[0].type ==
+            cls.tab_type)
 
 class VideoItemsDisplay(AudioVideoItemsDisplay):
-    @staticmethod
-    def should_display(tab_type, selected_tabs):
-        return tab_type == 'library' and selected_tabs[0].id == 'videos'
+    tab_type = u'videos'
+    tab_id = u'videos'
 
-    def make_controller(self, tab):
+    def make_controller(self):
         return itemlistcontroller.VideoItemsController()
 
 class AudioItemsDisplay(AudioVideoItemsDisplay):
-    @staticmethod
-    def should_display(tab_type, selected_tabs):
-        return tab_type == 'library' and selected_tabs[0].id == 'audios'
+    tab_type = u'music'
+    tab_id = u'music'
 
-    def make_controller(self, tab):
+    def make_controller(self):
         return itemlistcontroller.AudioItemsController()
 
 class OtherItemsDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'library' and selected_tabs[0].id == 'others'
+        return tab_type == 'others'
 
     def make_controller(self, tab):
         return itemlistcontroller.OtherItemsController()
@@ -371,26 +491,27 @@ class OtherItemsDisplay(ItemListDisplay):
 class DownloadingDisplay(ItemListDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'library' and selected_tabs[0].id == 'downloading'
+        return tab_type == 'downloading'
 
     def make_controller(self, tab):
         return downloadscontroller.DownloadsController()
 
-class VideoConversionsDisplay(TabDisplay):
+class ConvertingDisplay(TabDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
-        return tab_type == 'library' and selected_tabs[0].id == 'conversions'
+        return tab_type == 'converting'
 
     def __init__(self, tab_type, selected_tabs):
         Display.__init__(self)
-        self.controller = videoconversionscontroller.VideoConversionsController()
+        self.controller = convertingcontroller.ConvertingController()
         self.widget = self.controller.widget
 
 class FolderContentsDisplay(ItemListDisplayMixin, Display):
-    def __init__(self, info):
-        self.type = 'folder-contents'
+    def __init__(self, info, start_playing):
+        self.type = u'folder-contents'
         self.id = info.id
-        self.controller = itemlistcontroller.FolderContentsController(info)
+        self.controller = itemlistcontroller.FolderContentsController(info,
+                play_initial_list=start_playing)
         self.widget = self.controller.widget
         Display.__init__(self)
 
@@ -401,7 +522,7 @@ class CantPlayWidget(widgetset.SolidBackground):
         label = widgetset.Label(_(
             "%(appname)s can't play this file.  You may "
             "be able to open it with a different program",
-            {"appname": config.get(prefs.SHORT_APP_NAME)}
+            {"appname": app.config.get(prefs.SHORT_APP_NAME)}
             ))
         label.set_color((1, 1, 1))
         vbox.pack_start(label)
@@ -419,7 +540,8 @@ class CantPlayWidget(widgetset.SolidBackground):
         hbox = widgetset.HBox(spacing=12)
         reveal_button = widgetset.Button(_('Reveal File'))
         self.play_externally_button = widgetset.Button(_('Play Externally'))
-        self.play_externally_button.connect('clicked', self._on_play_externally)
+        self.play_externally_button.connect('clicked',
+                                            self._on_play_externally)
         skip_button = widgetset.Button(_('Skip'))
         reveal_button.connect('clicked', self._on_reveal)
         skip_button.connect('clicked', self._on_skip)
@@ -448,7 +570,7 @@ class CantPlayWidget(widgetset.SolidBackground):
         app.widgetapp.open_file(self.video_path)
 
     def _on_skip(self, button):
-        app.playback_manager.play_next_item(False)
+        app.playback_manager.play_next_item()
 
     def set_video_path(self, video_path):
         self.video_path = video_path
@@ -488,16 +610,21 @@ class VideoDisplay(Display):
         self.emit('ready-to-play')
 
     def _open_error(self):
-        messages.MarkItemWatched(self.item_info_id).send_to_backend()
+        if not self.item_info.video_watched:
+            messages.MarkItemWatched(self.item_info).send_to_backend()
         self.show_play_external()
         self.emit('cant-play')
 
-    def setup(self, item_info, volume):
+    def setup(self, item_info, item_type, volume):
         self.show_renderer()
         self.cant_play_widget.set_video_path(item_info.video_path)
-        self.item_info_id = item_info.id
-        self.renderer.set_item(item_info, self._open_success, self._open_error)
-        self.renderer.set_volume(volume)
+        self.item_info = item_info
+        if item_type != 'video':
+            self._open_error()
+        else:
+            self.renderer.set_item(item_info, self._open_success,
+                                   self._open_error)
+            self.renderer.set_volume(volume)
 
     def enter_fullscreen(self):
         self.renderer.enter_fullscreen()
@@ -522,7 +649,19 @@ class VideoDisplay(Display):
     def cleanup(self):
         if self._showing_renderer:
             self._prepare_remove_renderer()
-        self.renderer.teardown()
+        # FIXME
+        #
+        # This isn't just feel-good defensive programming.
+        #
+        # When the tab disappears abnormally and it is a video display it
+        # is destroyed.  That's fine.  However, it will happily try to
+        # select a new tab, which tries to remove the video display again
+        # because it is connected to 'removed' signal and calls
+        # on_display_removed.  So we may end up calling the cleanup
+        # twice.  I think the proper fix is to ensure that cleanup can only
+        # be called once, but right now it's not too bad hopefully.
+        if self.renderer:
+            self.renderer.teardown()
         self.renderer = None
 
 class MultipleSelectionDisplay(TabDisplay):
@@ -534,14 +673,7 @@ class MultipleSelectionDisplay(TabDisplay):
         Display.__init__(self)
         self.type = tab_type
         self.child_count = self.folder_count = self.folder_child_count = 0
-        if tab_type == 'feed':
-            tab_list = app.tab_list_manager.feed_list
-        elif tab_type == 'audio-feed':
-            tab_list = app.tab_list_manager.audio_feed_list
-        elif tab_type == 'site':
-            tab_list = app.tab_list_manager.site_list
-        else:
-            tab_list = app.tab_list_manager.playlist_list
+        tab_list = app.tabs[tab_type]
         for tab in selected_tabs:
             if hasattr(tab, "is_folder") and tab.is_folder:
                 self.folder_count += 1
@@ -550,7 +682,8 @@ class MultipleSelectionDisplay(TabDisplay):
                 self.child_count += 1
         vbox = widgetset.VBox(spacing=20)
         label = self._make_label(tab_type, selected_tabs)
-        label.set_size(2)
+        label.set_size(widgetutil.font_scale_from_osx_points(30))
+        label.set_bold(True)
         label.set_color((0.3, 0.3, 0.3))
         vbox.pack_start(widgetutil.align_center(label))
         vbox.pack_start(widgetutil.align_center(
@@ -562,15 +695,15 @@ class MultipleSelectionDisplay(TabDisplay):
         # NOTE: we need to use ngettext because some languages have multiple
         # plural forms.
         if self.folder_count > 0:
-            if tab_type in ('feed', 'audio-feed'):
+            if tab_type == 'feed':
                 label_parts.append(ngettext(
-                        '%(count)d Feed Folder Selected',
-                        '%(count)d Feed Folders Selected',
+                        '%(count)d Podcast Folder Selected',
+                        '%(count)d Podcast Folders Selected',
                         self.folder_count,
                         {"count": self.folder_count}))
                 label_parts.append(ngettext(
-                        '(contains %(count)d feed)',
-                        '(contains %(count)d feeds)',
+                        '(contains %(count)d podcast)',
+                        '(contains %(count)d podcasts)',
                         self.folder_child_count,
                         {"count": self.folder_child_count}))
             else:
@@ -581,23 +714,23 @@ class MultipleSelectionDisplay(TabDisplay):
                         {"count": self.folder_count}))
                 label_parts.append(ngettext(
                         '(contains %(count)d playlist)',
-                        '(contains %(count)d playlist)',
+                        '(contains %(count)d playlists)',
                         self.folder_child_count,
                         {"count": self.folder_child_count}))
 
         if self.child_count > 0 and self.folder_count > 0:
             label_parts.append('')
         if self.child_count > 0:
-            if tab_type in ('feed', 'audio-feed'):
+            if tab_type == 'feed':
                 label_parts.append(ngettext(
-                        '%(count)d Feed Selected',
-                        '%(count)d Feeds Selected',
+                        '%(count)d Podcast Selected',
+                        '%(count)d Podcasts Selected',
                         self.child_count,
                         {"count": self.child_count}))
             elif tab_type == "site":
                 label_parts.append(ngettext(
-                        '%(count)d Website Selected',
-                        '%(count)d Websites Selected',
+                        '%(count)d Source Selected',
+                        '%(count)d Sources Selected',
                         self.child_count,
                         {"count": self.child_count}))
             else:
@@ -621,21 +754,74 @@ class MultipleSelectionDisplay(TabDisplay):
         return hbox
 
     def _on_delete_clicked(self, button):
-        if self.type in ('feed', 'audio-feed'):
-            app.widgetapp.remove_current_feed()
-        elif self.type == 'site':
-            app.widgetapp.remove_current_site()
-        else:
-            app.widgetapp.remove_current_playlist()
+        app.widgetapp.remove_something()
+        iter_ = app.tabs[self.type].view.model.first_iter()
+        row = app.tabs[self.type].view.model[iter_]
+        root = app.tabs[self.type].get_tab(row[0].id)
+        app.display_manager.select_display_for_tabs(self.type, [root])
 
     def _on_new_folder_clicked(self, button):
-        if self.type in ('feed', 'audio-feed'):
-            section = {"feed": u"video", "audio-feed": u"audio"}
-            app.widgetapp.add_new_feed_folder(add_selected=True,
-                    default_type=self.type)
+        if self.type == 'feed':
+            app.widgetapp.add_new_feed_folder(add_selected=True)
         else:
             app.widgetapp.add_new_playlist_folder(add_selected=True)
 
+class ConnectDisplay(TabDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return tab_type == u'tab' and len(selected_tabs) == 1 and \
+               selected_tabs[0].tab_class == u'connect'
+
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+
+        self.widget = widgetset.Scroller(False, True)
+        alignment = widgetset.Alignment(xalign=0.5, yalign=0.0, xscale=1)
+        alignment.add(tabcontroller.ConnectTab())
+        self.widget.add(alignment)
+
+class SourcesDisplay(TabDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return (tab_type == u'tab' and len(selected_tabs) == 1 and
+                selected_tabs[0].tab_class == u'site')
+
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+
+        self.widget = widgetset.Scroller(False, True)
+        alignment = widgetset.Alignment(xalign=0.5, yalign=0.0, xscale=1)
+        alignment.add(tabcontroller.SourcesTab())
+        self.widget.add(alignment)
+
+class PlaylistsDisplay(TabDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return (tab_type == u'tab' and len(selected_tabs) == 1 and
+                selected_tabs[0].tab_class == u'playlist')
+
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+
+        self.widget = widgetset.Scroller(False, True)
+        alignment = widgetset.Alignment(xalign=0.5, yalign=0.0, xscale=1)
+        alignment.add(tabcontroller.PlaylistsTab())
+        self.widget.add(alignment)
+
+class StoresDisplay(TabDisplay):
+    @staticmethod
+    def should_display(tab_type, selected_tabs):
+        return (tab_type == u'tab' and len(selected_tabs) == 1 and
+                selected_tabs[0].tab_class == u'store')
+
+    def __init__(self, tab_type, selected_tabs):
+        Display.__init__(self)
+
+        self.widget = widgetset.Scroller(False, True)
+        alignment = widgetset.Alignment(xalign=0.5, yalign=0.0, xscale=1)
+        alignment.add(tabcontroller.StoresTab())
+        self.widget.add(alignment)
+
 class DummyDisplay(TabDisplay):
     @staticmethod
     def should_display(tab_type, selected_tabs):
