@@ -85,6 +85,12 @@ class FilteredListMixin(object):
         """Handle the filter switch changing state."""
         self.update_filters(filter_)
         app.widget_state.toggle_filters(self.type, self.id, filter_)
+        # Did we toggle a filter of the currently playing playlist
+        if ((self.current_item_view and 
+            app.playback_manager.is_playing and
+            self.current_item_view.model == 
+            app.playback_manager.playlist.model)):
+                app.playback_manager.reshuffle()
 
     def update_filters(self, filters):
         """Update the display and toolbar filter switch state."""
@@ -381,6 +387,9 @@ class ItemListController(object):
         self.widget.vbox[standard_view].pack_start(
                 standard_view_scroller, expand=True)
         self.views[standard_view].set_scroller(standard_view_scroller)
+        standard_view_scroller.set_background_color(
+                standard_view_widget.BACKGROUND_COLOR)
+        standard_view_scroller.prepare_for_dark_content()
 
         toolbar.connect_weak('sort-changed',
             self.on_sort_changed, standard_view)
