@@ -59,6 +59,7 @@ from miro.frontends.widgets import widgetutil
 from miro.frontends.widgets import dialogs
 from miro.frontends.widgets import widgetconst
 from miro.frontends.widgets import dialogwidgets
+from miro.frontends.widgets.widgetstatestore import WidgetStateStore
 from miro.plat import resources
 from miro.plat.utils import filename_to_unicode, get_logical_cpu_count
 from miro.plat.frontends.widgets.bonjour import install_bonjour
@@ -479,7 +480,8 @@ class GeneralPanel(PanelBuilder):
         v.pack_start(widgetutil.align_left(
             dialogwidgets.note(
                     _("(Changing the language requires you to "
-                      "restart Miro.)"))))
+                      "restart %(appname)s.)",
+                      {"appname": app.config.get(prefs.SHORT_APP_NAME)}))))
 
         pack_extras(v, "general")
 
@@ -526,6 +528,12 @@ class PodcastsPanel(PanelBuilder):
         attach_combo(max_option_menu, prefs.MAX_OLD_ITEMS_DEFAULT,
             [op[0] for op in max_options])
 
+        view_options = [(WidgetStateStore.STANDARD_VIEW, _("Standard view")),
+                      (WidgetStateStore.LIST_VIEW, _("List view"))]
+        view_option_menu = widgetset.OptionMenu([op[1] for op in view_options])
+        attach_combo(view_option_menu, prefs.PODCASTS_DEFAULT_VIEW,
+            [op[0] for op in view_options])
+
         grid.pack(dialogwidgets.heading(
                 _("Default settings for new podcasts:")),
                 grid.ALIGN_LEFT, span=2)
@@ -546,6 +554,11 @@ class PodcastsPanel(PanelBuilder):
         grid.pack(ad_option_menu)
         grid.end_line(spacing=4)
 
+        grid.pack_label(_("Default view:"),
+                dialogwidgets.ControlGrid.ALIGN_RIGHT)
+        grid.pack(view_option_menu)
+        grid.end_line(spacing=4)
+
         grid.pack(dialogwidgets.label_with_note(
             _("Remember this many old items:"),
             _("(in addition to the current contents)")),
@@ -958,7 +971,9 @@ class PlaybackPanel(PanelBuilder):
     def build_widget(self):
         v = widgetset.VBox()
 
-        miro_cbx = widgetset.Checkbox(_('Play media in Miro.'))
+        miro_cbx = widgetset.Checkbox(
+                    _('Play media in %(appname)s.',
+                      {"appname": app.config.get(prefs.SHORT_APP_NAME)}))
         separate_cbx = widgetset.Checkbox(
             _('Always play videos in a separate window.'))
 
