@@ -43,6 +43,12 @@ import BaseHTTPServer
 import SocketServer
 import threading
 import httplib
+from httplib import _CS_IDLE, _CS_REQ_STARTED, _CS_REQ_SENT, _UNKNOWN
+import gzip
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 # Where do I get this guy in Python?
 # NB: equivalent to INT32_MAX.
@@ -141,7 +147,6 @@ class DaapTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
 
     def daap_timeout_callback(self, s):
         self.del_session(s)
-        self.finished_callback(s)
 
     def session_count(self):
         return len(self.activeconn)
@@ -196,6 +201,8 @@ class DaapTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
                 del self.activeconn[s]
             except KeyError:
                 pass
+        if self.finished_callback:
+            self.finished_callback(s)
 
 class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     protocol_version = 'HTTP/1.1'
@@ -228,8 +235,8 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             pass
 
     def do_send_reply(self, rcode, reply, content_type=DEFAULT_CONTENT_TYPE,
-                      extra_headers=[]):
-        blob = encode_response(reply)
+                      content_encoding=None, extra_headers=[]):
+        blob = encode_response(reply, content_encoding=content_encoding)
         try:
             self.send_response(rcode)
             self.send_header('Content-type', content_type)
@@ -277,8 +284,6 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         reply.append(('minm', self.server_version))    # XXX FIXME
         reply.append(('mstm', DAAP_TIMEOUT))
 
-        # 'msup' not supported, but we don't indicate that by writing a 0.
-        # We do it by leaving it out.
         not_supported = [
                          'msix',    # Indexing
                          'msex',    # Extensions
@@ -290,7 +295,8 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                         ]
         supported = [
                      'msal',        # auto-logout
-                     'mslr'         # login
+                     'mslr',        # login
+                     'msup'         # on the fly updates
                     ]
         for code in not_supported:
             reply.append((code, 0))
@@ -345,17 +351,18 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
            return (DAAP_FORBIDDEN, [], [])
         self.server.del_session(session)
 
-    # We don't support this but Rhythmbox sends this anyway.  Grr.
     def do_update(self):
         path, query = split_url_path(self.path)
         session = self.get_session()
+        old_revision, old_delta = self.get_revision(query)
+        if not old_revision:
+            return (DAAP_BADREQUEST, [], [])
         if not session:
             return (DAAP_FORBIDDEN, [], [])
-        # UGH.  We should be updating this ... this is not supported at the 
-        # moment.
-        xxx_revision = 2
+        revision = self.server.backend.get_revision(session, old_revision,
+                                                    self.request)
         reply = []
-        reply.append(('mupd', [('mstt', DAAP_OK), ('musr', xxx_revision)]))
+        reply.append(('mupd', [('mstt', DAAP_OK), ('musr', revision)]))
         return (DAAP_OK, reply, [])
 
     def do_stream_file(self, db_id, item_id, ext, chunk):
@@ -400,12 +407,15 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         session = self.get_session()
         if not session:
             return (DAAP_FORBIDDEN, [], [])
+        revision, delta = self.get_revision(query)
         if len(path) == 1:
             reply = []
             db = []
-            count = len(self.server.backend.get_items())
+            items = self.server.backend.get_items()
+            count = len([i for i in items.values() if i['valid']])
             name = self.server.name
-            npl = 1 + len(self.server.backend.get_playlists())
+            playlists = self.server.backend.get_playlists()
+            npl = 1 + len([p for p in playlists.values() if p['valid']])
             db.append(('mlit', [
                                 ('miid', 1),    # Item ID
                                 ('mper', 1),    # Persistent ID
@@ -415,9 +425,10 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                                 # default playlist.
                                 ('mctc', npl)   # Playlist count
                                ]))
+            update = 1 if delta else 0
             reply.append(('avdb', [
                                    ('mstt', DAAP_OK),   # OK
-                                   ('muty', 0),         # Update type
+                                   ('muty', update),    # Update type
                                    ('mtco', 1),         # Specified total count
                                    ('mrco', 1),         # Returned count
                                    ('mlcl', db)         # db listing
@@ -440,21 +451,32 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     def _check_db_id(self, db_id):
         return db_id == 1
 
+    # NB: Should probably be a static class method.
+    def get_revision(self, query):
+        revision = delta = 0
+        try:
+            revision = int(query['revision-number'])
+            delta = int(query['delta'])
+        except (KeyError, ValueError):
+            pass
+        return revision, delta
+    
     # do_database_xxx(self, path, query): helper functions.  Session already
     # checked and we know we are in database/xxx.
     def do_database_containers(self, path, query):
         db_id = int(path[1])
         if not self._check_db_id(db_id):
             return (DAAP_FORBIDDEN, [], [])
+        revision, delta = self.get_revision(query)
         reply = []
         if len(path) == 3:
             # There is a requirement to send a default playlist so we
             # try to always send that one.
             count = len(self.server.backend.get_items())
             default_playlist = [('mlit', [
-                                          ('miid', 1),     # Item id
+                                          ('miid', 2),     # Item id
                                           ('minm', 'Library'),
-                                          ('mper', 1),     # Persistent id
+                                          ('mper', 2),     # Persistent id
                                           ('mimc', count), # count
                                           ('mpco', 0),     # parent containerid
                                           ('abpl', 1)      # Base playlist 
@@ -462,6 +484,7 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                                )]
             playlists = self.server.backend.get_playlists()
             playlist_list = []
+            deleted = []
             try:
                 meta = query['meta']
             except KeyError:
@@ -469,27 +492,36 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             meta_list = [m.strip() for m in meta.split(',')]
             for k in playlists.keys():
                 playlistprop = playlists[k]
-                playlist = []
-                for m in meta_list:
-                    if m in playlistprop.keys():
-                        try:
-                            code = dmap_consts_rmap[m]
-                        except KeyError:
-                            continue
-                        if playlistprop[m] is not None:
-                            attribute = (code, playlistprop[m])
-                            playlist.append(attribute)
-                playlist_list.append(('mlit', playlist))
+                if playlistprop['revision'] <= delta:
+                    continue
+                if playlistprop['valid']:
+                    playlist = []
+                    for m in meta_list:
+                        if m in playlistprop.keys():
+                            try:
+                                code = dmap_consts_rmap[m]
+                            except KeyError:
+                                continue
+                            if playlistprop[m] is not None:
+                                attribute = (code, playlistprop[m])
+                                playlist.append(attribute)
+                    playlist_list.append(('mlit', playlist))
+                else:
+                    deleted.append(('miid', k))
                                           
-            npl = 1 + len(playlists)
-            reply.append(('aply', [                   # Database playlists
-                                   ('mstt', DAAP_OK), # Status - OK
-                                   ('muty', 0),       # Update type
-                                   ('mtco', npl),     # total count
-                                   ('mrco', npl),     # returned count
-                                   ('mlcl', default_playlist + playlist_list)
-                                  ]
-                        ))
+            update = 1 if delta else 0
+            mlcl = default_playlist + playlist_list
+            npl = len(mlcl)
+            content = [                    # Database playlists
+                        ('mstt', DAAP_OK), # Status - OK
+                        ('muty', update),  # Update type
+                        ('mtco', npl),     # total count
+                        ('mrco', npl),     # returned count
+                        ('mlcl', mlcl)     # Playlist listing
+                       ]
+            if deleted:
+                content.append(('mudl', deleted))
+            reply.append(('aply', content))
         else:
             # len(path) > 3
             playlist_id = int(path[3])
@@ -513,47 +545,58 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         # on.  playlist_id is Library default so it if asks for that as a 
         # container (playlist) we still want to send the playlist version.
         backend_id = playlist_id
-        if backend_id == 1:
+        if backend_id == 2:
             backend_id = None
         items = self.server.backend.get_items(playlist_id=backend_id)
-        nfiles = len(items)
         itemlist = []
+        deleted = []
         try:
             meta = query['meta']
         except KeyError:
             meta = DEFAULT_DAAP_META
+        revision, delta = self.get_revision(query) 
         meta_list = [m.strip() for m in meta.split(',')]
         # NB: mikd must be the first guy in the listing.
         # GRR stupid Rhythmbox!  The meta reply must appear in order otherwise
         # it doesn't work!
         for k in items.keys():
             itemprop = items[k]
-            item = []
-            for m in meta_list:
-                if m in itemprop.keys():
-                    try:
-                        code = dmap_consts_rmap[m]
-                    except KeyError:
-                        continue
-                    if itemprop[m] is not None:
-                        attribute = (code, itemprop[m])
-                        item.append(attribute)
-            itemlist.append(('mlit', [       # Listing item
+            if itemprop['revision'] <= delta:
+                continue
+            if itemprop['valid']:
+                item = []
+                for m in meta_list:
+                    if m in itemprop.keys():
+                        try:
+                            code = dmap_consts_rmap[m]
+                        except KeyError:
+                            continue
+                        if itemprop[m] is not None:
+                            attribute = (code, itemprop[m])
+                            item.append(attribute)
+                itemlist.append(('mlit', [       # Listing item
                                       # item kind - seems OK to hardcode this.
-                                      ('mikd', DAAP_ITEMKIND_AUDIO),
-                                     ] + item
+                                          ('mikd', DAAP_ITEMKIND_AUDIO),
+                                         ] + item
                            )) 
- 
+            else:
+                deleted.append(('miid', k))
+
         tag = 'apso' if playlist_id else 'adbs'
+        nfiles = len(itemlist)
+        update = 1 if delta else 0
         reply = []
-        reply = [(tag, [                     # Container type
+        content = [                          # Container type
                         ('mstt', DAAP_OK),   # Status: OK
-                        ('muty', 0),         # Update type
+                        ('muty', update),    # Update type
                         ('mtco', nfiles),    # Specified total count
                         ('mrco', nfiles),    # Returned count
-                        ('mlcl', itemlist)   # Itemlist container
-                       ]
-                )]
+                        ('mlcl', itemlist)
+                  ]
+        if deleted:
+            content.append(('mudl', deleted))    # Itemlist deleted
+
+        reply = [(tag, content)]
         return (DAAP_OK, reply, [])
 
     def do_database_items(self, path, query):
@@ -640,10 +683,51 @@ class DaapHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             rcode = DAAP_BADREQUEST
             reply = []
             extra_headers = []
+        content_encoding = self.reply_encoding()
+        self.do_send_reply(rcode, reply, extra_headers=extra_headers,
+                           content_encoding=content_encoding)
         if endconn:
             self.wfile.close()
-        else:
-            self.do_send_reply(rcode, reply, extra_headers=extra_headers)
+
+    def reply_encoding(self):
+        supported = ['gzip']
+        header = self.headers.getheader('Accept-encoding')
+        if not header:
+            return None
+        acceptable = header.split(',')
+        candidates = []
+        prohibited = []
+        for x in acceptable:
+            try:
+                encoding = x.split(',')
+                encoding, qvalue = x.split(',')
+                qvalue = float(qvalue)
+            except ValueError:
+                encoding = encoding[0]
+                qvalue = None
+            encoding = encoding.strip()
+            if qvalue == 0:
+                prohibited.append(encoding)
+            else:
+                candidates.append((encoding, qvalue))
+        def sort(x, y):
+           # NB: None > 0 == False, None < 0 == False
+           _, q1 = x
+           _, q2 = y
+           if x > y:
+               return 1
+           elif x < y:
+               return -1
+           else:
+               return 0
+        candidates.sort(cmp=sort, reverse=True)
+        for e, _ in candidates:
+            if e in supported:
+                return e
+        # XXX what to do if the 'identity' encoding or the wildcard was
+        # was administratively prohibited?  We can check for '*' in the
+        # prohibited list.
+        return None
 
 def mdns_init():
     return mdns.mdns_init()
@@ -746,11 +830,17 @@ def make_daap_server(backend, debug=False, name='pydaap', port=DEFAULT_PORT,
 # HTTP/1.1.
 class DaapClient(object):
     HEARTBEAT = 60    # seconds
-    def __init__(self, host, port):
+    def __init__(self, host, port, gzip=False):
         self.conn = None
         self.host = host
         self.port = port
+        self.gzip = gzip
         self.session = None
+        self.headers = dict()
+        self.old_revision = self.revision = 1
+        self.supports_update = False
+        if self.gzip:
+           self.headers['Accept-encoding'] = 'gzip, identity'
 
     def heartbeat_callback(self):
         try:
@@ -779,14 +869,24 @@ class DaapClient(object):
         # XXX Broken - don't do an unbounded read here, this is stupid,
         # server can crash the client
         data = response.read()
+        encoding = response.getheader('Content-encoding')
+        if encoding is not None and encoding.strip() == 'gzip':
+            data = gzip.GzipFile(fileobj=StringIO(data)).read()
         if callback:
             callback(data, *args)
 
+    def handle_server_info(self, data):
+        update = find_daap_tag('msup', decode_response(data))
+        self.supports_update = True if update else False
+
     def handle_login(self, data):
         self.session = find_daap_tag('mlid', decode_response(data))
 
     # Note: in theory there could be multiple DB but in reality there's only
     # one.  So this is a shortcut.
+    #
+    # XXX in theory we may have to handle database being deleted, or new
+    # database being added.
     def handle_db(self, data):
         db_list = find_daap_tag('mlcl', decode_response(data))
         # Just get the first one.
@@ -794,38 +894,55 @@ class DaapClient(object):
         self.db_id = find_daap_tag('miid', db)
         self.db_name = find_daap_tag('minm', db)
 
+    def handle_update(self, data):
+        revision = find_daap_tag('musr', decode_response(data))
+        self.old_revision = self.revision
+        self.revision = revision
+
     def handle_playlist(self, data, meta):
-        listing = find_daap_tag('mlcl', decode_response(data))
+        r = decode_response(data)
+        listing = find_daap_tag('mlcl', r)
+        deleted = find_daap_tag('mudl', r)
         playlist_dict = dict()
+        deleted_list = []
         meta_list = [m.strip() for m in meta.split(',')]
-        for item in find_daap_listitems(listing):
-            playlist_id = find_daap_tag('miid', item)
-            playlist_dict[playlist_id] = dict()
-            for m in meta_list:
-                try:
-                    playlist_dict[playlist_id][m] = find_daap_tag(
+        if listing is not None:
+            for item in find_daap_listitems(listing):
+                playlist_id = find_daap_tag('miid', item)
+                playlist_dict[playlist_id] = dict()
+                for m in meta_list:
+                    try:
+                        playlist_dict[playlist_id][m] = find_daap_tag(
                                                     dmap_consts_rmap[m], item)
-                except KeyError:
-                    continue
-        self.daap_playlists = playlist_dict
+                    except KeyError:
+                        continue
+        if deleted is not None:
+            for item_id in find_daap_listitems(deleted):
+                deleted_list.append(item_id)
+
+        self.daap_playlists = (playlist_dict, deleted_list)
 
     def handle_items(self, data, playlist_id, meta):
-        listing = find_daap_tag('mlcl', decode_response(data))
-        itemdict = dict()
-        if not listing:
-            self.daap_items = dict()    # dummy empty
-            return
+        r = decode_response(data)
+        listing = find_daap_tag('mlcl', r)
+        deleted = find_daap_tag('mudl', r)
         meta_list = [m.strip() for m in meta.split(',')]
-        for item in find_daap_listitems(listing):
-            itemid = find_daap_tag('miid', item)
-            itemdict[itemid] = dict()
-            for m in meta_list:
-                try:
-                    itemdict[itemid][m] = find_daap_tag(
-                                          dmap_consts_rmap[m], item)
-                except KeyError:
-                    continue
-        self.daap_items = itemdict
+        itemdict = dict()
+        deleted_list = []
+        if listing is not None:
+            for item in find_daap_listitems(listing):
+                itemid = find_daap_tag('miid', item)
+                itemdict[itemid] = dict()
+                for m in meta_list:
+                    try:
+                        itemdict[itemid][m] = find_daap_tag(
+                                              dmap_consts_rmap[m], item)
+                    except KeyError:
+                        continue
+        if deleted is not None:
+            for item_id in find_daap_listitems(deleted):
+                deleted_list.append(item_id)
+        self.daap_items = itemdict, deleted_list
 
     def sessionize(self, request, query):
         if not self.session:
@@ -833,19 +950,22 @@ class DaapClient(object):
         new_request = request + '?session-id=%d' % self.session
         # XXX urllib.quote?
         new_request = '&'.join([new_request] + 
-                               [name + '=' + param for name, param in query])
+                               [name + '=' + str(param) for
+                                name, param in query])
         return new_request
 
     def connect(self):
         try:
             self.conn = httplib.HTTPConnection(self.host, self.port)
-            self.conn.request('GET', '/server-info')
-            self.check_reply(self.conn.getresponse())            
-            self.conn.request('GET', '/content-codes')
+            self.conn.request('GET', '/server-info', headers=self.headers)
+            self.check_reply(self.conn.getresponse(),
+                             callback=self.handle_server_info)
+            self.conn.request('GET', '/content-codes', headers=self.headers)
             self.check_reply(self.conn.getresponse())
-            self.conn.request('GET', '/login')
+            self.conn.request('GET', '/login', headers=self.headers)
             self.check_reply(self.conn.getresponse(),
                              callback=self.handle_login)
+            self.update()
             # Finally, if this all works, start the heartbeat timer.
             # XXX pick out the daap timeout from the server.
             self.timer = threading.Timer(self.HEARTBEAT,
@@ -857,29 +977,61 @@ class DaapClient(object):
             self.disconnect()
             return False
         except httplib.BadStatusLine:
-            self.disconnect(polite=False)
+            self.disconnect()
             return False
 
     # XXX Right now, there is only one db_id.
-    def databases(self):
+    # XXX Right now, can't handle deleted database listing (I don't think that
+    # ever happens.)
+    def databases(self, update=False):
         try:
-            self.conn.request('GET', self.sessionize('/databases', []))
+            revquery = self.revision_query(update)
+            self.conn.request('GET', self.sessionize('/databases', revquery),
+                              headers=self.headers)
             self.check_reply(self.conn.getresponse(),
                              callback=self.handle_db)
             return self.db_id
         # We've been disconnected or there was a problem?
-        except (socket.error, IOError, ValueError):
+        except (AttributeError, socket.error, IOError, ValueError,
+                httplib.BadStatusLine):
             self.disconnect()
             return None
-        except httplib.BadStatusLine:
-            self.disconnect(polite=False)
+
+    def revision_query(self, update):
+        if not self.supports_update:
+            return []
+        query = [('revision-number', self.revision)]
+        if update:
+            query += [('delta', self.old_revision)]
+        return query
+
+    def update(self):
+        if not self.supports_update:
+            return
+        try:
+            # Setting True/False here is not very relevant, since delta is not
+            # used.
+            #
+            # NOTE: iTunes uses a revision-number= but no delta= on first go,
+            # all subsequent ones have revision-number= and delta= set to be
+            # the same number.
+            revquery = self.revision_query(False)
+            self.conn.request('GET', self.sessionize('/update', revquery),
+                              headers=self.headers)
+            self.check_reply(self.conn.getresponse(),
+                             callback=self.handle_update)
+        except (AttributeError, httplib.BadStatusLine, socket.error,
+                IOError, ValueError):
+            self.disconnect()
             return None
 
-    def playlists(self, meta=DEFAULT_DAAP_PLAYLIST_META):
+    def playlists(self, meta=DEFAULT_DAAP_PLAYLIST_META, update=False):
         try:
+            revquery = self.revision_query(update)
             self.conn.request('GET', self.sessionize(
                               '/databases/%d/containers' % self.db_id,
-                              [('meta', meta)]))
+                              [('meta', meta)] + revquery),
+                              headers=self.headers)
             self.check_reply(self.conn.getresponse(),
                              callback=self.handle_playlist,
                              args=[meta])
@@ -887,27 +1039,26 @@ class DaapClient(object):
             del self.daap_playlists
             return playlists
         # We've been disconnected or there was a problem?
-        except (socket.error, IOError, ValueError):
+        except (AttributeError, socket.error, IOError, ValueError,
+                httplib.BadStatusLine):
             self.disconnect()
             return None
-        except httplib.BadStatusLine:
-            self.disconnect(polite=False)
-            return None
 
     # XXX: I think this could be cleaner, maybe abstract to have an
     # easy way to provide the daap meta without resorting to providing
     # the raw string which includes the names requested.
-    def items(self, playlist_id=None, meta=DEFAULT_DAAP_META):
+    def items(self, playlist_id=None, meta=DEFAULT_DAAP_META, update=False):
         try:
+            query = self.revision_query(update) + [('meta', meta)]
             if playlist_id is None:
                 self.conn.request('GET', self.sessionize(
                     '/databases/%d/items' % self.db_id,
-                    [('meta', meta)]))
+                    query), headers=self.headers)
             else:
                 self.conn.request('GET', self.sessionize(
                     ('/databases/%d/containers/%d/items' % 
                      (self.db_id, playlist_id)),
-                    [('meta', meta)]))
+                    query), headers=self.headers)
             self.check_reply(self.conn.getresponse(),
                              callback=self.handle_items,
                              args=[playlist_id, meta])
@@ -915,22 +1066,18 @@ class DaapClient(object):
             del self.daap_items
             return items
         # We've been disconnected or there was a problem?
-        except (socket.error, IOError, ValueError):
-            typ, value, tb = sys.exc_info()
-            print 'items: typ = %s value = %s' % (str(typ), str(value))
-            for line in traceback.format_tb(tb):
-                print line
+        except (AttributeError, socket.error, IOError, ValueError,
+                httplib.BadStatusLine):
             self.disconnect()
             return None
-        except httplib.BadStatusLine:
-            self.disconnect(polite=False)
-            return None
 
-    def disconnect(self, polite=True):
+    def disconnect(self):
         try:
             self.timer.cancel()
-            if polite:
-                self.conn.request('GET', self.sessionize('/logout', []))
+            # Don't use - you are just making life difficult for yourself to
+            # nuke the existing connection ...
+            #if polite:
+            #    self.conn.request('GET', self.sessionize('/logout', []))
         # Don't care since we are going away anyway.
         except (socket.error, ValueError, httplib.ResponseNotReady,
                 httplib.BadStatusLine, AttributeError, IOError):
@@ -940,8 +1087,10 @@ class DaapClient(object):
             # block doesn't raise exception but just prints it out.
             self.session = None
             if self.conn:
-                self.conn.close()
+                conn = self.conn
                 self.conn = None
+                conn.sock.shutdown(socket.SHUT_RDWR)
+                conn.close()
 
     def daap_get_file_request(self, file_id, enclosure=None):
         """daap_file_get_url(file_id) -> url
@@ -957,8 +1106,8 @@ class DaapClient(object):
         fn += '?session-id=%s' % self.session
         return fn
 
-def make_daap_client(host, port=DEFAULT_PORT):
-    return DaapClient(host, port)
+def make_daap_client(host, port=DEFAULT_PORT, gzip=True):
+    return DaapClient(host, port, gzip=gzip)
 
 def register_meta(meta, code, typ):
     dmap_consts[code] = (meta, typ)
