@@ -27,8 +27,8 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-"""``miro.messages`` -- Message passing between the frontend thread
-and the backend thread.
+"""``miro.messages`` -- Message passing between the frontend thread and the
+backend thread.
 
 The backend thread is the eventloop, which processes things like feed
 updates and handles reading and writing to the database.  The frontend
@@ -36,15 +36,14 @@ thread is the thread of the UI toolkit we're using.  Communication
 between the two threads is handled by passing messages between the
 two.  These messages are handled asynchronously.
 
-This module defines the messages that are passed between the two
-threads.
+This module defines the messages that are passed between the two threads.
 """
 
 import logging
-import re
 
 from miro.gtcache import gettext as _
 from miro.folder import ChannelFolder, PlaylistFolder
+from miro.messagetools import Message, MessageHandler
 from miro.plat import resources
 from miro import app
 from miro import displaytext
@@ -53,62 +52,6 @@ from miro import search
 from miro import prefs
 from miro import util
 
-class MessageHandler(object):
-    def __init__(self):
-        self.message_map = {} # maps message classes to method names
-        self.complained_about = set()
-
-    def call_handler(self, method, message):
-        """Arrange for a message handler method to be called in the correct
-        thread.  Must be implemented by subclasses.
-        """
-        raise NotImplementedError()
-
-    def handle(self, message):
-        """Handles a given message.
-        """
-        handler_name = self.get_message_handler_name(message)
-        try:
-            handler = getattr(self, handler_name)
-        except AttributeError:
-            if handler_name not in self.complained_about:
-                logging.warn("MessageHandler doesn't have a %s method "
-                        "to handle the %s message" % (handler_name,
-                            message.__class__))
-                self.complained_about.add(handler_name)
-        else:
-            self.call_handler(handler, message)
-
-    def get_message_handler_name(self, message):
-        try:
-            return self.message_map[message.__class__]
-        except KeyError:
-            self.message_map[message.__class__] = \
-                    self.calc_message_handler_name(message.__class__)
-            return self.message_map[message.__class__]
-
-    def calc_message_handler_name(self, message_class):
-        def replace(match):
-            return '%s_%s' % (match.group(1), match.group(2))
-        underscores = re.sub(r'([a-z])([A-Z])', replace,
-                message_class.__name__)
-        return 'handle_' + util.ascii_lower(underscores)
-
-class Message(object):
-    """Base class for all Messages.
-    """
-    @classmethod
-    def install_handler(cls, handler):
-        """Install a new message handler for this class.  When
-        send_to_frontend() or send_to_backend() is called, this handler will
-        be invoked.
-        """
-        cls.handler = handler
-
-    @classmethod
-    def reset_handler(cls):
-        del cls.handler
-
 class BackendMessage(Message):
     """Base class for Messages that get sent to the backend.
     """
@@ -1220,20 +1163,20 @@ class ItemInfo(object):
     html_stripper = util.HTMLStripper()
 
     def __repr__(self):
-        return "<ItemInfo %s>" % self.id
+        return "<ItemInfo %r>" % self.id
 
     def __getstate__(self):
         d = self.__dict__.copy()
         d['device'] = None
         del d['description_stripped']
-        del d['search_ngrams']
+        del d['search_terms']
         return d
 
     def __setstate__(self, d):
         self.__dict__.update(d)
         self.description_stripped = ItemInfo.html_stripper.strip(
                 self.description)
-        self.search_ngrams = search.calc_ngrams(self)
+        self.search_terms = search.calc_search_terms(self)
 
     def __init__(self, id_, **kwargs):
         self.id = id_
@@ -1245,8 +1188,8 @@ class ItemInfo(object):
         if not hasattr(self, 'description_stripped'):
             self.description_stripped = ItemInfo.html_stripper.strip(
                 self.description)
-        if not hasattr(self, 'search_ngrams'):
-            self.search_ngrams = search.calc_ngrams(self)
+        if not hasattr(self, 'search_terms'):
+            self.search_terms = search.calc_search_terms(self)
         self.name_sort_key = util.name_sort_key(self.name)
         self.album_sort_key = util.name_sort_key(self.album)
         self.artist_sort_key = util.name_sort_key(self.artist)
@@ -1816,6 +1759,7 @@ class GlobalInfo(object):
     def __init__(self, global_info):
         self.item_details_expanded = global_info.item_details_expanded
         self.guide_sidebar_expanded = global_info.guide_sidebar_expanded
+        self.tabs_width = global_info.tabs_width
 
 class ViewInfo(object):
     """Contains the properties that are unique to each View
