@@ -119,6 +119,7 @@ MUSIC_TAB_CLICKED           = Pref(key='musicTabClicked',       default=False, p
 SHOW_PODCASTS_IN_VIDEO      = Pref(key='showPodcastsInVideo', default=True, platformSpecific=False)
 SHOW_PODCASTS_IN_MUSIC      = Pref(key='showPodcastsInMusic', default=False, platformSpecific=False)
 REMEMBER_LAST_DISPLAY       = Pref(key='rememberLastDisplay', default=False, platformSpecific=False)
+PODCASTS_DEFAULT_VIEW       = Pref(key='podcastsDefaultView', default=0, platformSpecific=False)
 # This doesn't need to be defined on the platform, but it can be overridden there if the platform wants to.
 SHOW_ERROR_DIALOG           = Pref(key='showErrorDialog',       default=True,  platformSpecific=True)
 
@@ -287,3 +288,8 @@ OPEN_FOLDER_ON_STARTUP = \
     Pref(key='openFolderOnStartup',default=None, platformSpecific=False)
 OPEN_CHANNEL_ON_STARTUP = \
     Pref(key='openChannelOnStartup',default=None, platformSpecific=False)
+DISABLE_IPV6 = \
+    Pref(key='disableIPV6',default=True, platformSpecific=False)
+
+def all_prefs():
+    return [obj for obj in globals().values() if isinstance(obj, Pref)]
