@@ -284,6 +284,7 @@ class SharingTracker(object):
 
     def try_to_add(self, share_id, fullname, host, port, uuid):
         def success(unused):
+            logging.debug('SUCCESS!!')
             if self.available_shares.has_key(share_id):
                 info = self.available_shares[share_id]
             else:
@@ -296,6 +297,7 @@ class SharingTracker(object):
             messages.TabsChanged('connect', [info], [], []).send_to_frontend()
 
         def failure(unused):
+            logging.debug('FAILURE')
             if self.available_shares.has_key(share_id):
                 info = self.available_shares[share_id]
             else:
@@ -497,9 +499,13 @@ class SharingTracker(object):
         self.thread.start()
 
     def eject(self, share_id):
-        tracker = self.trackers[share_id]
-        del self.trackers[share_id]
-        tracker.client_disconnect()
+        try:
+            tracker = self.trackers[share_id]
+        except KeyError:
+            pass
+        else:
+            del self.trackers[share_id]
+            tracker.client_disconnect()
 
     def get_tracker(self, share_id):
         try:
@@ -555,6 +561,9 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
     """
     type = u'sharing'
     def __init__(self, share):
+        signals.SignalEmitter.__init__(self)
+        for sig in 'added', 'changed', 'removed':
+            self.create_signal(sig)
         self.client = None
         self.share = share
         self.items = dict()
@@ -564,13 +573,48 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         self.share.is_updating = True
         message = messages.TabsChanged('connect', [], [self.share], [])
         message.send_to_frontend()
-        eventloop.call_in_thread(self.client_connect_callback,
-                                 self.client_connect_error_callback,
-                                 self.client_connect,
-                                 'DAAP client connect')
-        signals.SignalEmitter.__init__(self)
-        for sig in 'added', 'changed', 'removed':
-            self.create_signal(sig)
+        name = self.share.name
+        host = self.share.host
+        port = self.share.port
+        title = 'Sharing Client %s @ (%s, %s)' % (name, host, port)
+        self.thread = threading.Thread(target=self.runloop,
+                                       name=title)
+        self.thread.daemon = True
+        self.thread.start()
+
+    def run(self, func, success, failure):
+        succeeded = False
+        try:
+            result = func()
+        except KeyboardInterrupt:
+            raise
+        except Exception, e:
+                logging.debug('>>> Exception %s %s', self.thread.name,
+                              ''.join(traceback.format_exc()))
+                func = failure
+                name = 'error callback (%s)' % self.thread.name
+                args = (e,)
+        else:
+                func = success
+                name = 'result callback (%s)' % self.thread.name
+                args = (result,)
+                succeeded = True
+        eventloop.add_idle(func, name, args=args)
+        return succeeded
+
+    def runloop(self):
+        success = self.run(self.client_connect, self.client_connect_callback,
+                           self.client_connect_error_callback)
+        # If server does not support update, then we short circuit since
+        # the loop becomes useless.  There is nothing wait for being updated.
+        logging.debug('UPDATE SUPPORTED = %s', self.client.supports_update)
+        if not success or not self.client.supports_update:
+            return
+        while True:
+            success = self.run(self.client_update, self.client_update_callback,
+                               self.client_update_error_callback)
+            if not success:
+                break
 
     def sharing_item(self, rawitem):
         kwargs = dict()
@@ -623,17 +667,21 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         client = self.client
         self.client = None
         playlist_ids = [playlist_.id for playlist_ in self.playlists]
-        message = messages.TabsChanged('connect', [], [], playlist_ids)
-        message.send_to_frontend()
         eventloop.call_in_thread(self.client_disconnect_callback,
                                  self.client_disconnect_error_callback,
                                  client.disconnect,
                                  'DAAP client connect')
 
     def client_disconnect_error_callback(self, unused):
+        playlist_ids = [playlist_.id for playlist_ in self.playlists]
+        message = messages.TabsChanged('connect', [], [], playlist_ids)
+        message.send_to_frontend()
         pass
 
     def client_disconnect_callback(self, unused):
+        playlist_ids = [playlist_.id for playlist_ in self.playlists]
+        message = messages.TabsChanged('connect', [], [], playlist_ids)
+        message.send_to_frontend()
         pass
 
     def client_connect(self):
@@ -652,19 +700,45 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         # Lousy Windows and Python API.
         address, port = self.client.conn.sock.getpeername()
         self.address = address
-        if not self.client.databases():
+        return self.setup_items()
+
+    # See use of self.client in client_update().
+    def setup_items(self, update=False):
+        name = self.share.name
+        host = self.share.host
+        port = self.share.port
+        try:
+            client = self.client
+        except AttributeError:
+            # Doesn't matter what exception it is, just raise to call error
+            # callback
+            raise
+
+        # From this point on ... if use of client is invalid (because socket
+        # is closed or something it should raise an error and we can bail
+        # out that way, and call the error callback.
+        if not client.databases(update=update):
             raise IOError('Cannot get database')
-        playlists = self.client.playlists()
+        deleted_items = dict()
+        playlists, deleted_playlists = client.playlists(update=update)
         if playlists is None:
             raise IOError('Cannot get playlist')
         returned_playlists = []
         for k in playlists.keys():
+            # Clean the playlist: remove NUL characters.
+            for k_ in playlists[k]:
+                if isinstance(playlists[k][k_], str):
+                    tmp = playlists[k][k_]
+                    playlists[k][k_] = tmp.replace('\x00', '')
+
             is_base_playlist = None
             if playlists[k].has_key('daap.baseplaylist'):
                 is_base_playlist = playlists[k]['daap.baseplaylist']
             if is_base_playlist:
-                if self.base_playlist:
+                if not update and self.base_playlist:
                     logging.debug('WARNING: more than one base playlist found')
+                if update and self.base_playlist != k:
+                    logging.debug('WARNING: base playlistid changed in update')
                 self.base_playlist = k
             # This isn't the playlist id of the remote share, this is the
             # playlist id we use internally.
@@ -688,32 +762,37 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         audio_playlist_id = unicode(md5(repr((name,
                                               host,
                                               port, u'audio'))).hexdigest())
-        video_info = messages.SharingInfo(video_playlist_id,
-                                          u'video',
-                                          host,
-                                          port,
-                                          parent_id=self.share.id,
-                                          playlist_id=u'video')
-        audio_info = messages.SharingInfo(audio_playlist_id,
-                                          u'audio',
-                                          host,
-                                          port,
-                                          parent_id=self.share.id,
-                                          playlist_id=u'audio')
-        # Place this stuff at the front
-        returned_playlists.insert(0, audio_info)
-        returned_playlists.insert(0, video_info)
+
+        # These are fake: so we only want to insert these once.
+        if not update:
+            video_info = messages.SharingInfo(video_playlist_id,
+                                              u'video',
+                                              host,
+                                              port,
+                                              parent_id=self.share.id,
+                                              playlist_id=u'video')
+            audio_info = messages.SharingInfo(audio_playlist_id,
+                                              u'audio',
+                                              host,
+                                              port,
+                                              parent_id=self.share.id,
+                                              playlist_id=u'audio')
+            # Place this stuff at the front
+            returned_playlists.insert(0, audio_info)
+            returned_playlists.insert(0, video_info)
 
         # Maybe we have looped through here without a base playlist.  Then
         # the server is broken?
         if not self.base_playlist:
             raise ValueError('Cannot find base playlist')
 
-        items = self.client.items(playlist_id=self.base_playlist,
-                                  meta=DAAP_META)
+        items, deleted = client.items(playlist_id=self.base_playlist,
+                                  meta=DAAP_META, update=update)
         if items is None:
             raise ValueError('Cannot find items in base playlist')
 
+        deleted_items[self.base_playlist] = deleted
+
         itemdict = dict()
         returned_playlist_items = dict()
         returned_items = []
@@ -724,6 +803,11 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         audio_items_meth = audio_items.append
         video_items_meth = video_items.append
         for itemkey in items.keys():
+            # Clean it of NUL
+            for k in items[itemkey]:
+                if isinstance(items[itemkey][k], str):
+                    tmp = items[itemkey][k]
+                    items[itemkey][k] = tmp.replace('\x00', '')
             item = sharing_item_meth(items[itemkey])
             itemdict[itemkey] = item
             returned_items_meth(item)
@@ -745,9 +829,11 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
                 continue
             returned_items = []
             returned_items_meth = returned_items.append
-            items = self.client.items(playlist_id=k, meta=DAAP_META)
+            items, deleted = client.items(playlist_id=k, meta=DAAP_META,
+                                               update=update)
             if items is None:
                 raise ValueError('Cannot find items for playlist %d' % k)
+            deleted_items[k] = deleted
             for itemkey in items.keys():
                 item = itemdict[itemkey]
                 returned_items_meth(item)
@@ -755,17 +841,132 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
 
         # We don't append these items directly to the object and let
         # the success callback to do it to prevent race.
-        return (returned_playlist_items, returned_playlists)
+        return (returned_playlist_items, returned_playlists,
+                deleted_playlists, deleted_items)
+
+    # If we are disconnecting, then, disconnect() sets the self.client
+    # to None before actually running the client.disconnect() routine.
+    # So, usage of self.client should be:
+    #
+    # try:
+    #    client = self.client
+    # except AttributeError:
+    #    # Handle error here.
+    #    pass
+    # else:
+    #     client.blah()
+    def client_update(self):
+        logging.debug('CLIENT UPDATE')
+        try:
+            client = self.client
+        except AttributeError:
+            # Doesn't matter what exception it is, just raise to call error
+            # callback
+            raise
+        client.update()
+        return self.setup_items(update=True)
+
+    def client_update_callback(self, args):
+        logging.debug('CLIENT UPDATE CALLBACK')
+        deleted = []
+        changed = []
+        added = []
+        (returned_items, returned_playlists,
+         deleted_playlists, deleted_items) = args
+
+        # Do the playlists first.  So when when we add items, it will
+        # Just Work.
+        playlist_dict = dict()
+        # XXX
+        for p in returned_playlists:
+            playlist_dict[p.playlist_id] = p
+        to_remove = []
+        playlist_ids = [p.playlist_id for p in self.playlists]
+        for p in self.playlists:
+            if p.playlist_id in deleted_playlists:
+                if not p.playlist_id in playlist_dict:
+                    deleted.append(p.id)
+                else:
+                    logging.debug('client update: weird server playlist id %s '
+                                  'in deleted and item list at the same time',
+                                  p.playlist_id)
+            if p.playlist_id in playlist_dict:
+                changed.append(playlist_dict[p.playlist_id])
+                to_remove.append(p)
+        added = [playlist_dict[p] for p in playlist_dict if p not in
+                 playlist_ids]
+        for p in to_remove:
+            self.playlists.remove(p)
+            # Only remove it if it's really going!
+            changed_ids = [p.playlist_id for p in changed]
+            if not p.playlist_id in changed_ids:
+                del self.items[p.playlist_id]
+        # Make sure that when we add something it is accessible.
+        for p in added:
+            self.items[p.playlist_id] = []
+        self.playlists += added
+        self.playlists += changed
+
+        for k in deleted_items:
+            item_ids = deleted_items[k]
+            try:
+                playlist_items = self.items[k]
+            except KeyError:
+                # Huh what?  Sent us something that we don't have anymore.
+                logging.debug('Playlist %s already deleted', k)
+                continue
+            to_remove = []
+            for item_id in item_ids:
+                for item in playlist_items:
+                    if item.id != item_id:
+                        continue
+                    to_remove.append(item)
+                    self.emit('removed', k, item)
+                    break
+            for r in to_remove:
+                playlist_items.remove(r)
+
+        for k in returned_items:
+            candidates = returned_items[k]
+            try:
+                playlist_items = self.items[k]
+            except KeyError:
+                logging.debug('CANNOT ACCESS self.items[%s]', k)
+                raise
+            to_remove = []
+            to_add = []
+            for candidate in candidates:
+                for item in playlist_items:
+                    if candidate.id == item.id:
+                        self.emit('changed', k, candidate)
+                        to_remove.append(item)
+                        to_add.append(candidate)
+                        break
+                else:
+                    self.emit('added', k, candidate)
+                    to_add.append(candidate)
+            for r in to_remove:
+                playlist_items.remove(r)
+            playlist_items += to_add
+
+        # Finally, update the tabs.
+        message = messages.TabsChanged('connect', added, changed, deleted)
+        message.send_to_frontend()
+
+    def client_update_error_callback(self, unused):
+        self.client_connect_update_error_callback(unused, update=True)
 
     # NB: this runs in the eventloop (backend) thread.
     def client_connect_callback(self, args):
-        returned_items, returned_playlists = args
+        # Just ignore any deleted items on first connect - they shouldn't
+        # be there.
+        returned_items, returned_playlists, _, _ = args
         self.items = returned_items
         self.playlists = returned_playlists
         # Send a list of all the items to the main sharing tab.  Only add
         # those that are part of the base playlist.
         for item in self.items[self.base_playlist]:
-            self.emit('added', item)
+            self.emit('added', None, item)
         # Once all the items are added then send display mounted and remove
         # the progress indicator.
         self.share.mount = True
@@ -775,11 +976,21 @@ class SharingItemTrackerImpl(signals.SignalEmitter):
         message.send_to_frontend()
 
     def client_connect_error_callback(self, unused):
+        self.client_connect_update_error_callback(unused)
+
+    def client_connect_update_error_callback(self, unused, update=False):
         # If it didn't work, immediately disconnect ourselves.
-        self.share.is_updating = False
-        message = messages.TabsChanged('connect', [], [self.share], [])
-        message.send_to_frontend()
-        app.sharing_tracker.eject(self.share.id)
+        # Non atomic test-and-do check ok - always in eventloop.
+        if self.client is None:
+            # someone already did handy-work for us - probably a disconnect
+            # happened while we were in the middle of an update().
+            return
+        if not update:
+            self.share.is_updating = False
+            message = messages.TabsChanged('connect', [], [self.share], [])
+            message.send_to_frontend()
+        if not self.share.stale_callback:
+            app.sharing_tracker.eject(self.share.id)
         messages.SharingConnectFailed(self.share).send_to_frontend()
 
     def get_items(self, playlist_id=None):
@@ -803,18 +1014,21 @@ class SharingManagerBackend(object):
     id = u'sharing-backend'
 
     def __init__(self):
+        self.revision = 1
         self.share_types = []
         if app.config.get(prefs.SHARE_AUDIO):
             self.share_types += [libdaap.DAAP_MEDIAKIND_AUDIO]
         if app.config.get(prefs.SHARE_VIDEO):
             self.share_types += [libdaap.DAAP_MEDIAKIND_VIDEO]
         self.item_lock = threading.Lock()
+        self.revision_cv = threading.Condition(self.item_lock)
         self.transcode_lock = threading.Lock()
         self.transcode = dict()
         # XXX daapplaylist should be hidden from view. 
         self.daapitems = dict()         # DAAP format XXX - index via the items
         self.daap_playlists = dict()    # Playlist, in daap format
         self.playlist_item_map = dict() # Playlist -> item mapping
+        self.deleted_item_map = dict()  # Playlist -> deleted item mapping
         self.in_shutdown = False
         self.config_handle = app.backend_config_watcher.connect('changed',
                              self.on_config_changed)
@@ -824,7 +1038,19 @@ class SharingManagerBackend(object):
         pass
 
     def handle_item_list(self, message):
-        self.make_item_dict(message.items)
+        with self.item_lock:
+            self.update_revision()
+            item_ids = [item.id for item in message.items]
+            if message.id is not None:
+                self.daap_playlists[message.id]['revision'] = self.revision
+                self.playlist_item_map[message.id] = item_ids
+                self.deleted_item_map[message.id] = []
+                # Update the revision of these items, so they will match
+                # when the playlist items are fetched.
+                for item_id in item_ids:
+                    self.daapitems[item_id]['revision'] = self.revision
+            else:
+                self.make_item_dict(message.items)
 
     def handle_items_changed(self, message):
         # If items are changed, overwrite with a recreated entry.  This
@@ -832,21 +1058,54 @@ class SharingManagerBackend(object):
         # item being moved out of, and then into, a playlist.  Also, based on 
         # message.id, change the playlists accordingly.
         with self.item_lock:
+            self.update_revision()
             for itemid in message.removed:
                 try:
                     if message.id is not None:
+                        revision = self.revision
+                        self.daap_playlists[message.id]['revision'] = revision
                         self.playlist_item_map[message.id].remove(itemid)
+                        self.deleted_item_map[message.id].append(itemid)
+                        logging.debug('INACTIVE %s %s', itemid, message.id)
                 except KeyError:
                     pass
                 try:
-                    del self.daapitems[itemid]
+                    if message.id is None:
+                        self.daapitems[itemid] = self.deleted_item()
                 except KeyError:
                     pass
             if message.id is not None:
                 item_ids = [item.id for item in message.added]
+                self.daap_playlists[message.id]['revision'] = self.revision
+                # If they have been previously removed, unmark deleted.
+                for i in item_ids:
+                    try:
+                        self.deleted_item_map[message.id].remove(i)
+                    except ValueError:
+                        pass
                 self.playlist_item_map[message.id] += item_ids
-            self.make_item_dict(message.added)
-            self.make_item_dict(message.changed)
+
+            # Only make or modify an item if it is for main library.
+            # Otherwise, we just re-create an item when all that's changed
+            # is the contents of the playlist.
+            if message.id is None:
+                self.make_item_dict(message.added)
+                self.make_item_dict(message.changed)
+            else:
+                # Simply update the item's revision.
+                for x in message.added:
+                    self.daapitems[x.id]['revision'] = self.revision
+                for x in message.changed:
+                    self.daapitems[x.id]['revision'] = self.revision
+
+    def deleted_item(self):
+        return dict(revision=self.revision, valid=False)
+
+    # At this point: item_lock acquired
+    def update_revision(self, directed=None):
+        self.revision += 1
+        self.directed = directed
+        self.revision_cv.notify_all()
 
     def make_daap_playlists(self, items):
         for item in items:
@@ -880,6 +1139,11 @@ class SharingManagerBackend(object):
             daap_string = 'dmap.persistentid'
             if daap_string == 'dmap.persistentid':
                 itemprop[daap_string] = item.id
+
+            # piece de resistance
+            itemprop['revision'] = self.revision
+            itemprop['valid'] = True
+
             self.daap_playlists[item.id] = itemprop
 
     def handle_playlist_added(self, obj, added):
@@ -887,7 +1151,20 @@ class SharingManagerBackend(object):
 
         def _handle_playlist_added():
             with self.item_lock:
+                self.update_revision()
                 self.make_daap_playlists(playlists)
+                for p in playlists:
+                    # no need to update the revision here: already done in
+                    # make_daap_playlists.
+                    self.playlist_item_map[p.id] = []
+                    self.deleted_item_map[p.id] = []
+                    app.info_updater.item_list_callbacks.add(self.type,
+                                                     p.id,
+                                                     self.handle_item_list)
+                    app.info_updater.item_changed_callbacks.add(self.type,
+                                                     p.id,
+                                                     self.handle_items_changed)
+                    messages.TrackItems(self.type, p.id).send_to_backend()
 
         eventloop.add_urgent_call(lambda: _handle_playlist_added(),
                                   "SharingManagerBackend: playlist added")
@@ -895,11 +1172,13 @@ class SharingManagerBackend(object):
     def handle_playlist_changed(self, obj, changed):
         def _handle_playlist_changed():
             with self.item_lock:
+                self.update_revision()
                 # We could just overwrite everything without actually deleting
                 # the object.  A missing key means it's a folder, and we skip
                 # over it.
                 for x in changed:
                     if self.daap_playlists.has_key(x.id):
+                        #self.daap_playlists[x.id] = self.deleted_item()
                         del self.daap_playlists[x.id]
                 playlist = [x for x in changed if not x.is_folder]
                 self.make_daap_playlists(playlist)
@@ -911,36 +1190,56 @@ class SharingManagerBackend(object):
     def handle_playlist_removed(self, obj, removed):
         def _handle_playlist_removed():
             with self.item_lock:
+                self.update_revision()
                 for x in removed:
                     # Missing key means it's a folder and we skip over it.
                     if self.daap_playlists.has_key(x):
-                        del self.daap_playlists[x]
+                        self.daap_playlists[x] = self.deleted_item()
+                        #del self.daap_playlists[x]
+                        del self.playlist_item_map[x]
+                        del self.deleted_item_map[x]
+                        messages.StopTrackingItems(self.type,
+                                                   x).send_to_backend()
+                        app.info_updater.item_list_callbacks.remove(self.type,
+                                                    x,
+                                                    self.handle_item_list)
+                        app.info_updater.item_changed_callbacks.remove(
+                                                    self.type,
+                                                    x,
+                                                    self.handle_items_changed)
 
         eventloop.add_urgent_call(lambda: _handle_playlist_removed(),
                                   "SharingManagerBackend: playlist removed")
 
+    # XXX I think we can probably do away with this one, since the item list
+    # callbacks will end up populating this anyway?
     def populate_playlists(self):
         with self.item_lock:
+            self.update_revision()
             self.make_daap_playlists(playlist.SavedPlaylist.make_view())
             for playlist_id in self.daap_playlists.keys():
+                # revision for playlist already created in make_daap_playlist
                 self.playlist_item_map[playlist_id] = [x.item_id
                   for x in playlist.PlaylistItemMap.playlist_view(playlist_id)]
+                self.deleted_item_map[playlist_id] = []
 
     def start_tracking(self):
         self.populate_playlists()
+        # Track items that do not belong in any playlist.  Do this first
+        # so we pick up all items in the media library.
+        app.info_updater.item_list_callbacks.add(self.type, None,
+                                                 self.handle_item_list)
+        app.info_updater.item_changed_callbacks.add(self.type, None,
+                                                 self.handle_items_changed)
+        messages.TrackItems(self.type, None).send_to_backend()
+
+        # Now, for the specific playlists.
         for playlist_id in self.daap_playlists:
             app.info_updater.item_list_callbacks.add(self.type, playlist_id,
                                                  self.handle_item_list)
             app.info_updater.item_changed_callbacks.add(self.type, playlist_id,
                                                  self.handle_items_changed)
             messages.TrackItems(self.type, playlist_id).send_to_backend()
-        # Track items that do not belong in any playlist.
-        app.info_updater.item_list_callbacks.add(self.type, None,
-                                                 self.handle_item_list)
-        app.info_updater.item_changed_callbacks.add(self.type, None,
-                                                 self.handle_items_changed)
-
-        messages.TrackItems(self.type, None).send_to_backend()
 
         app.info_updater.connect('playlists-added',
                                  self.handle_playlist_added)
@@ -968,6 +1267,38 @@ class SharingManagerBackend(object):
         app.info_updater.disconnect(self.handle_playlist_changed)
         app.info_updater.disconnect(self.handle_playlist_removed)
 
+    def watcher(self, session, request):
+        while True:
+            try:
+                r, w, x = select.select([request], [], [])
+                # Unlock the revision by bumping it
+                with self.item_lock:
+                    logging.debug('WAKEUP %s', session)
+                    self.update_revision(directed=session)
+                break
+            except select.error, (err, errstring):
+                if err == errno.EINTR:
+                    continue
+            except StandardError, err:
+                raise ValueError('watcher: unknown error during select')
+
+    def get_revision(self, session, old_revision, request):
+        self.revision_cv.acquire()
+        while self.revision == old_revision:
+            t = threading.Thread(target=self.watcher, args=(session, request))
+            t.daemon = True
+            t.start()
+            self.revision_cv.wait()
+            # If we really did a update or if the wakeup was directed at us
+            # (because we are quitting or something) then release the lock
+            # and return the revision
+            if self.directed is None or self.directed == session:
+                break
+            # update revision and then wait again
+            old_revision = self.revision
+        self.revision_cv.release()
+        return self.revision
+
     def get_file(self, itemid, generation, ext, session, request_path_func,
                  offset=0, chunk=None):
         file_obj = None
@@ -1063,11 +1394,21 @@ class SharingManagerBackend(object):
         keys = [prefs.SHARE_AUDIO.key, prefs.SHARE_VIDEO.key]
         if key in keys:
             with self.item_lock:
+                share_types_orig = self.share_types
                 self.share_types = []
                 if app.config.get(prefs.SHARE_AUDIO):
                     self.share_types += [libdaap.DAAP_MEDIAKIND_AUDIO]
                 if app.config.get(prefs.SHARE_VIDEO):
                     self.share_types += [libdaap.DAAP_MEDIAKIND_VIDEO]
+                # Just by enabling and disabing this, the selection of items
+                # available to a user could have changed.
+                #
+                if share_types_orig != self.share_types:
+                    self.update_revision()
+                for p in self.daap_playlists:
+                    self.daap_playlists[p]['revision'] = self.revision
+                for i in self.daapitems:
+                    self.daapitems[i]['revision'] = self.revision
 
     def get_items(self, playlist_id=None):
         # Easy: just return
@@ -1075,17 +1416,24 @@ class SharingManagerBackend(object):
             items = dict()
             if not playlist_id:
                 for k in self.daapitems.keys():
-                    if (self.daapitems[k]['com.apple.itunes.mediakind'] in
-                      self.share_types):
-                        items[k] = self.daapitems[k]
+                    item = self.daapitems[k]
+                    if (not item['valid'] or
+                      item['com.apple.itunes.mediakind'] in self.share_types):
+                        items[k] = item
+                    else:
+                        items[k] = self.deleted_item()
                 return items
             # XXX Somehow cache this?
             playlist = dict()
-            for x in self.daapitems.keys():
-                if (x in self.playlist_item_map[playlist_id] and
-                  self.daapitems[x]['com.apple.itunes.mediakind'] in
-                  self.share_types):
-                    playlist[x] = self.daapitems[x]
+            if self.playlist_item_map.has_key(playlist_id):
+                for x in self.daapitems.keys():
+                    item = self.daapitems[x]
+                    if (x in self.playlist_item_map[playlist_id] and
+                      (not item['valid'] or
+                      item['com.apple.itunes.mediakind'] in self.share_types)):
+                        playlist[x] = item
+                    else:
+                        playlist[x] = self.deleted_item()
             return playlist
 
     def make_item_dict(self, items):
@@ -1119,7 +1467,7 @@ class SharingManagerBackend(object):
             # out the container.  We can change it if that's no longer true.
             # Fixup the media kind: XXX what about u'other'?
             enclosure = item.file_format
-            if not enclosure:
+            if enclosure not in supported_filetypes:
                 nam, ext = os.path.splitext(item.video_path)
                 if ext in supported_filetypes:
                     enclosure = ext
@@ -1134,27 +1482,17 @@ class SharingManagerBackend(object):
             if itemprop['com.apple.itunes.mediakind'] == u'video':
                 itemprop['com.apple.itunes.mediakind'] = (
                   libdaap.DAAP_MEDIAKIND_VIDEO)
-                if not itemprop['daap.songformat']:
-                    if not enclosure:
-                        enclosure = '.mp4'
-                    enclosure = enclosure[1:]
-                    itemprop['daap.songformat'] = enclosure
-                else:
-                    # make sure we strip away the .dot
-                    enclosure = itemprop['daap.songformat'][1:]
-                    itemprop['daap.songformat'] = enclosure
+                if not enclosure:
+                    enclosure = '.mp4'
+                enclosure = enclosure[1:]
+                itemprop['daap.songformat'] = enclosure
             else:
                 itemprop['com.apple.itunes.mediakind'] = (
                   libdaap.DAAP_MEDIAKIND_AUDIO)
-                if not itemprop['daap.songformat']:
-                    if not enclosure:
-                        enclosure = '.mp3'
-                    enclosure = enclosure[1:]
-                    itemprop['daap.songformat'] = enclosure
-                else:
-                    # make sure we strip away the .dot
-                    enclosure = itemprop['daap.songformat'][1:]
-                    itemprop['daap.songformat'] = enclosure
+                if not enclosure:
+                    enclosure = '.mp3'
+                enclosure = enclosure[1:]
+                itemprop['daap.songformat'] = enclosure
             # Normally our strings are fixed up above, but then we re-pull
             # this out of the input data structure, so have to re-convert.
             if isinstance(itemprop['daap.songformat'], unicode):
@@ -1165,6 +1503,15 @@ class SharingManagerBackend(object):
             # ok: it is ignored since this is not valid dmap/daap const.
             itemprop['path'] = item.video_path
             itemprop['cover_art'] = item.thumbnail
+
+            # HACK: the rmapping dict doesn't work because we can't
+            # double up the key.
+            itemprop['dmap.containeritemid'] = itemprop['dmap.itemid']
+
+            # piece de resistance: tack on the revision.
+            itemprop['revision'] = self.revision
+            itemprop['valid'] = True
+
             self.daapitems[item.id] = itemprop
 
     def finished_callback(self, session):
