@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,12 +38,12 @@ import os
 from miro.frontends.widgets import widgetconst
 from miro.plat import resources
 from miro.plat.frontends.widgets import wrappermap
-from miro.plat.frontends.widgets import layoutmanager
 from miro.plat.frontends.widgets.base import Widget
 from miro.plat.frontends.widgets.helpers import NotificationForwarder
 
 from miro import searchengines
-from miro import config, prefs
+from miro import app
+from miro import prefs
 
 class SizedControl(Widget):
     def set_size(self, size):
@@ -53,7 +54,8 @@ class SizedControl(Widget):
             font = NSFont.systemFontOfSize_(NSFont.smallSystemFontSize())
             self.view.cell().setControlSize_(NSSmallControlSize)
         else:
-            raise ValueError("Unknown size: %s" % size)
+            self.view.cell().setControlSize_(NSRegularControlSize)
+            font = NSFont.systemFontOfSize_(NSFont.systemFontSize() * size)
         self.view.setFont_(font)
 
 class BaseTextEntry(SizedControl):
@@ -80,11 +82,21 @@ class BaseTextEntry(SizedControl):
         self.create_signal('validate')
 
     def focus(self):
-        self.view.window().makeFirstResponder_(self.view)
+        if self.view.window() is not None:
+            self.view.window().makeFirstResponder_(self.view)
+
+    def start_editing(self, initial_text):
+        self.set_text(initial_text)
+        self.focus()
+        # unselect the text and locate the cursor at the end of the entry
+        text_field = self.view.window().fieldEditor_forObject_(YES, self.view)
+        text_field.setSelectedRange_(NSMakeRange(len(self.get_text()), 0))
 
     def viewport_created(self):
         SizedControl.viewport_created(self)
         self.notifications.connect(self.on_changed, 'NSControlTextDidChangeNotification')
+        self.notifications.connect(self.on_end_editing,
+                'NSControlTextDidEndEditingNotification')
 
     def remove_viewport(self):
         SizedControl.remove_viewport(self)
@@ -96,6 +108,9 @@ class BaseTextEntry(SizedControl):
     def on_changed(self, notification):
         self.emit('changed')
 
+    def on_end_editing(self, notification):
+        self.emit('focus-out')
+
     def calc_size_request(self):
         size = self.sizer_cell.cellSize()
         return size.width, size.height
@@ -123,34 +138,65 @@ class BaseTextEntry(SizedControl):
         self.view.setEnabled_(False)
 
 class MiroTextField(NSTextField):
-    def becomeFirstResponder(self):
+    def textDidEndEditing_(self, notification):
         wrappermap.wrapper(self).emit('activate')
-        return NSTextField.becomeFirstResponder(self)
+        return NSTextField.textDidEndEditing_(self, notification)
 
 class TextEntry(BaseTextEntry):
     def make_view(self):
         return MiroTextField.alloc().init()
 
+class NumberEntry(BaseTextEntry):
+    def make_view(self):
+        return MiroTextField.alloc().init()
+
+    def set_max_length(self, length):
+        # TODO
+        pass
+
+    def _filter_value(self):
+        """Discard any non-numeric characters"""
+        digits = ''.join(x for x in self.view.stringValue() if x.isdigit())
+        self.view.setStringValue_(digits)
+
+    def on_changed(self, notification):
+        # overriding on_changed rather than connecting to it ensures that we
+        # filter the value before anything else connected to the signal sees it
+        self._filter_value()
+        BaseTextEntry.on_changed(self, notification)
+
+    def get_text(self):
+        # handles get_text between when text is entered and when on_changed
+        # filters it, in case that's possible
+        self._filter_value()
+        return BaseTextEntry.get_text(self)
+
 class MiroSecureTextField(NSSecureTextField):
-    def becomeFirstResponder(self):
+    def textDidEndEditing_(self, notification):
         wrappermap.wrapper(self).emit('activate')
-        return NSSecureTextField.becomeFirstResponder(self)
+        return NSSecureTextField.textDidEndEditing_(self, notification)
 
 class SecureTextEntry(BaseTextEntry):
     def make_view(self):
         return MiroSecureTextField.alloc().init()
 
 class MiroSearchTextField(NSSearchField):
-    def becomeFirstResponder(self):
+    def init(self):
+        self = super(MiroSearchTextField, self).init()
+        image = NSImage.alloc().initByReferencingFile_(resources.path('images/search_icon_all.png'))
+        self.cell().searchButtonCell().setImage_(image)
+        self.cell().searchButtonCell().setAlternateImage_(image)
+        return self
+
+    def textDidEndEditing_(self, notification):
         wrappermap.wrapper(self).emit('activate')
-        return NSSearchField.becomeFirstResponder(self)
+        return NSSearchField.textDidEndEditing_(self, notification)
 
 class SearchTextEntry(BaseTextEntry):
     def make_view(self):
         return MiroSearchTextField.alloc().init()
 
 class MultilineTextEntry(Widget):
-
     def __init__(self, initial_text=None):
         Widget.__init__(self)
         if initial_text is None:
@@ -160,6 +206,8 @@ class MultilineTextEntry(Widget):
         self.view.setHorizontallyResizable_(NO)
         self.view.setVerticallyResizable_(YES)
         self.notifications = NotificationForwarder.create(self.view)
+        self.create_signal('changed')
+        self.create_signal('focus-out')
         if initial_text is not None:
             self.set_text(initial_text)
         self.set_size(widgetconst.SIZE_NORMAL)
@@ -176,6 +224,8 @@ class MultilineTextEntry(Widget):
     def viewport_created(self):
         Widget.viewport_created(self)
         self.notifications.connect(self.on_changed, 'NSTextDidChangeNotification')
+        self.notifications.connect(self.on_end_editing,
+                'NSControlTextDidEndEditingNotification')
         self.invalidate_size_request()
 
     def remove_viewport(self):
@@ -183,7 +233,8 @@ class MultilineTextEntry(Widget):
         self.notifications.disconnect()
 
     def focus(self):
-        self.view.window().makeFirstResponder_(self.view)
+        if self.view.window() is not None:
+            self.view.window().makeFirstResponder_(self.view)
 
     def set_text(self, text):
         self.view.setString_(text)
@@ -194,15 +245,25 @@ class MultilineTextEntry(Widget):
 
     def on_changed(self, notification):
         self.invalidate_size_request()
+        self.emit("changed")
+
+    def on_end_editing(self, notification):
+        self.emit("focus-out")
 
     def calc_size_request(self):
-        if self.view.superview() is None:
-            return (50, 50)
-        width = self.view.superview().frame().size.width
-        height = self.view.frame().size.height
-        if self.parent_is_scroller:
-            width -= NSScroller.scrollerWidth()
-        return (width, height)
+        layout_manager = self.view.layoutManager()
+        text_container = self.view.textContainer()
+        # The next line is there just to force cocoa to layout the text
+        layout_manager.glyphRangeForTextContainer_(text_container)
+        rect = layout_manager.usedRectForTextContainer_(text_container)
+        return rect.size.width, rect.size.height
+
+    def set_editable(self, editable):
+        if editable:
+            self.view.setEditable_(YES)
+        else:
+            self.view.setEditable_(NO)
+
     
 class MiroButton(NSButton):
     
@@ -219,13 +280,14 @@ class MiroButton(NSButton):
 
 class Checkbox(SizedControl):
     """See https://develop.participatoryculture.org/index.php/WidgetAPI for a description of the API for this class."""
-    def __init__(self, label):
+    def __init__(self, text="", bold=False):
         SizedControl.__init__(self)
         self.create_signal('toggled')
-        self.label = label
         self.view = MiroButton.alloc().initWithSignal_('toggled')
         self.view.setButtonType_(NSSwitchButton)
-        self.view.setTitle_(self.label)
+        self.view.setTitle_(text)
+        if bold:
+            self.view.setFont_(NSFont.boldSystemFontOfSize_(0))
 
     def calc_size_request(self):
         size = self.view.cell().cellSize()
@@ -251,9 +313,16 @@ class Checkbox(SizedControl):
         SizedControl.disable(self)
         self.view.setEnabled_(False)
 
+    def get_text_padding(self):
+        """
+        Returns the amount of space the checkbox takes up before the label.
+        """
+        # XXX FIXME
+        return 18
+
 class Button(SizedControl):
     """See https://develop.participatoryculture.org/index.php/WidgetAPI for a description of the API for this class."""
-    def __init__(self, label, style='normal'):
+    def __init__(self, label, style='normal', width=0):
         SizedControl.__init__(self)
         self.color = None
         self.title = label
@@ -262,7 +331,7 @@ class Button(SizedControl):
         self.view.setButtonType_(NSMomentaryPushInButton)
         self._set_title()
         self.setup_style(style)
-        self.min_width = 0
+        self.min_width = width
 
     def set_text(self, label):
         self.title = label
@@ -360,6 +429,10 @@ class OptionMenu(SizedControl):
         SizedControl.disable(self)
         self.view.setEnabled_(False)
 
+    def set_width(self, width):
+        # TODO
+        pass
+
 class RadioButtonGroup:
     def __init__(self):
         self._buttons = []
@@ -454,9 +527,39 @@ class NSVideoSearchField (MiroSearchTextField):
         self.cell().setSendsWholeSearchString_(YES)
         self.cell().setSendsSearchStringImmediately_(NO)
         self.cell().setScrollable_(YES)
+        # handle the user clicking cancel ourselves
+        self.cell().cancelButtonCell().setTarget_(self)
+        self.cell().cancelButtonCell().setAction_("handleCancel:")
         self.setStringValue_("")
         return self
 
+    # FIXME: ticket #17143 has several improvements for this code
+
+    def handleCancel_(self, sender):
+        self.setStringValue_('')
+        # emit 'validate' here to clear the search
+        wrappermap.wrapper(self).emit('validate')
+
+    def textDidChange_(self, notification):
+        # NOTE: we purposely avoid doing anything here.  The Cocoa class calls
+        # our action method when the textbox is empty, but this causes issues
+        # (#17129, comment 5).
+        pass
+
+    def textDidEndEditing_(self, notification):
+        # Handle the user stopping typing.
+        text_movement = notification.userInfo().get('NSTextMovement')
+        if text_movement == NSReturnTextMovement:
+            # user hit return, emit 'validate'
+            wrappermap.wrapper(self).emit('validate')
+
+        # HACK: not sure exactly why this line has an effect, but it prevents
+        # 17195
+        self.stringValue()
+
+        # Don't call superclass method.  The cocoa class hides the cancel
+        # button if the text is blank, but we want to override that behavior.
+
     def makeSearchMenuTemplate(self):
         menu = NSMenu.alloc().initWithTitle_("Search Menu")
         for engine in reversed(searchengines.get_search_engines()):
@@ -477,8 +580,7 @@ class NSVideoSearchField (MiroSearchTextField):
         self.cell().searchButtonCell().setImage_(_getSearchIcon(engine))
 
     def search_(self, sender):
-        if self.stringValue() != "":
-            wrappermap.wrapper(self).emit('validate')
+        wrappermap.wrapper(self).emit('validate')
 
     def menuItemForEngine_(self, engine):
         return self._engineToMenuItem[engine]
@@ -504,13 +606,13 @@ NSVideoSearchField.setCellClass_(VideoSearchFieldCell)
 
 def _getEngineIcon(engine):
     engineIconPath = resources.path('images/search_icon_%s.png' % engine.name)
-    if config.get(prefs.THEME_NAME) and engine.filename:
+    if app.config.get(prefs.THEME_NAME) and engine.filename:
         if engine.filename.startswith(resources.theme_path(
-            config.get(prefs.THEME_NAME), 'searchengines')):
+            app.config.get(prefs.THEME_NAME), 'searchengines')):
                 # this search engine came from a theme; look up the icon in the
                 # theme directory instead
                 engineIconPath = resources.theme_path(
-                    config.get(prefs.THEME_NAME),
+                    app.config.get(prefs.THEME_NAME),
                     'images/search_icon_%s.png' % engine.name)
     if not os.path.exists(engineIconPath):
         return nil
