@@ -1,5 +1,6 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2010 Participatory Culture Foundation
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+# Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,16 +27,6 @@
 # this exception statement from your version. If you delete this exception
 # statement from all source files in the program, then also delete it here.
 
-from AppKit import *
-from Foundation import *
-from objc import YES, NO, nil
-
-from miro.frontends.widgets import imagepool
-
-from miro.plat import resources
-from miro.plat.frontends.widgets import widgetset
-from miro.plat.frontends.widgets.rect import Rect
-
 def get_platform_specific(name):
     pass
 
